"""
关系管理器

负责管理实体间的关系映射：
- Agent-Service 关系
- Service-Tool 关系
"""

import logging
import time
from typing import Any, Dict, List, TYPE_CHECKING, Set

from .models import (
    AgentServiceRelation,
    ServiceRelationItem,
    ServiceToolRelation,
    ToolRelationItem
)

if TYPE_CHECKING:
    from .cache_layer_manager import CacheLayerManager

logger = logging.getLogger(__name__)


class RelationshipManager:
    """
    关系管理器
    
    管理实体间的关系映射，包括：
    - Agent-Service 关系（key 是 agent_id）
    - Service-Tool 关系（key 是 service_global_name）
    """
    
    def __init__(self, cache_layer: 'CacheLayerManager'):
        """
        初始化关系管理器
        
        Args:
            cache_layer: 缓存层管理器实例
        """
        self._cache_layer = cache_layer
        # 仅记录一次的调试日志（例如每个服务第一次读取关系）
        self._service_tools_logged_once: Set[str] = set()
        # 用于统计某个服务关系缺失的次数，便于做聚合日志
        self._service_relation_missing_counts: Dict[str, int] = {}
        # 记录上一次每个服务的工具数量，仅在数量变化时输出调试日志
        self._last_service_tool_counts: Dict[str, int] = {}
        logger.debug("[RELATIONSHIP] Initializing RelationshipManager")
    
    # ==================== Agent-Service 关系管理 ====================
    
    async def add_agent_service(
        self,
        agent_id: str,
        service_original_name: str,
        service_global_name: str,
        client_id: str
    ) -> None:
        """
        添加 Agent-Service 关系
        
        Args:
            agent_id: Agent ID
            service_original_name: 服务原始名称
            service_global_name: 服务全局名称
            client_id: 客户端 ID
            
        Raises:
            ValueError: 如果参数无效
            KeyError: 如果服务实体不存在
            RuntimeError: 如果添加失败
        """
        if not agent_id:
            raise ValueError("Agent ID cannot be empty")
        if not service_original_name:
            raise ValueError("Service original name cannot be empty")
        if not service_global_name:
            raise ValueError("Service global name cannot be empty")
        if not client_id:
            raise ValueError("Client ID cannot be empty")
        
        # 验证服务实体存在
        service_entity = await self._cache_layer.get_entity(
            "services",
            service_global_name
        )
        if service_entity is None:
            raise KeyError(
                f"Service entity does not exist: service_global_name={service_global_name}"
            )
        
        logger.debug(
            f"[RELATIONSHIP] Adding Agent-Service relation: agent_id={agent_id}, "
            f"service_original_name={service_original_name}, "
            f"service_global_name={service_global_name}, client_id={client_id}"
        )
        
        # 获取现有关系
        relation_data = await self._cache_layer.get_relation(
            "agent_services",
            agent_id
        )
        
        if relation_data is None:
            # 创建新关系
            relation = AgentServiceRelation(services=[])
        else:
            # 解析现有关系
            relation = AgentServiceRelation.from_dict(relation_data)
        
        # 检查服务是否已存在（基于全局名称判断）
        for i, service in enumerate(relation.services):
            if service.service_global_name == service_global_name:
                # 全局名称相同，认为是同一个关系，更新配置
                relation.services[i] = ServiceRelationItem(
                    service_original_name=service_original_name,
                    service_global_name=service_global_name,
                    client_id=client_id,
                    established_time=service.established_time,
                    last_access=int(time.time())
                )
                
                await self._cache_layer.put_relation(
                    "agent_services",
                    agent_id,
                    relation.to_dict()
                )
                
                logger.info(
                    f"[RELATIONSHIP] Updated Agent-Service relation: agent_id={agent_id}, "
                    f"service_global_name={service_global_name}"
                )
                return
        
        # 添加新服务
        current_time = int(time.time())
        new_service = ServiceRelationItem(
            service_original_name=service_original_name,
            service_global_name=service_global_name,
            client_id=client_id,
            established_time=current_time,
            last_access=current_time
        )
        relation.services.append(new_service)
        
        # 保存关系
        await self._cache_layer.put_relation(
            "agent_services",
            agent_id,
            relation.to_dict()
        )
        
        logger.info(
            f"[RELATIONSHIP] Successfully added Agent-Service relation: agent_id={agent_id}, "
            f"service_global_name={service_global_name}"
        )
    
    async def remove_agent_service(
        self,
        agent_id: str,
        service_global_name: str
    ) -> None:
        """
        移除 Agent-Service 关系
        
        Args:
            agent_id: Agent ID
            service_global_name: 服务全局名称
            
        Raises:
            ValueError: 如果参数无效
            KeyError: 如果关系不存在
            RuntimeError: 如果移除失败
        """
        if not agent_id:
            raise ValueError("Agent ID cannot be empty")
        if not service_global_name:
            raise ValueError("Service global name cannot be empty")
        
        logger.debug(
            f"[RELATIONSHIP] Removing Agent-Service relation: agent_id={agent_id}, "
            f"service_global_name={service_global_name}"
        )
        
        # 获取现有关系
        relation_data = await self._cache_layer.get_relation(
            "agent_services",
            agent_id
        )
        
        if relation_data is None:
            raise KeyError(
                f"Agent relation does not exist: agent_id={agent_id}"
            )
        
        # 解析关系
        relation = AgentServiceRelation.from_dict(relation_data)
        
        # 查找并移除服务
        original_count = len(relation.services)
        relation.services = [
            service for service in relation.services
            if service.service_global_name != service_global_name
        ]
        
        if len(relation.services) == original_count:
            raise KeyError(
                f"Service does not exist in Agent relation: agent_id={agent_id}, "
                f"service_global_name={service_global_name}"
            )
        
        # 保存更新后的关系
        if len(relation.services) == 0:
            # 如果没有服务了，删除整个关系
            await self._cache_layer.delete_relation("agent_services", agent_id)
            logger.info(
                f"[RELATIONSHIP] Deleted empty Agent relation: agent_id={agent_id}"
            )
        else:
            # 保存更新后的关系
            await self._cache_layer.put_relation(
                "agent_services",
                agent_id,
                relation.to_dict()
            )
            logger.info(
                f"[RELATIONSHIP] Successfully removed Agent-Service relation: "
                f"agent_id={agent_id}, service_global_name={service_global_name}"
            )
    
    async def get_agent_services(
        self,
        agent_id: str
    ) -> List[Dict[str, Any]]:
        """
        获取 Agent 的所有服务关系
        
        Args:
            agent_id: Agent ID
            
        Returns:
            服务关系列表，如果不存在返回空列表
            
        Raises:
            ValueError: 如果参数无效
            RuntimeError: 如果获取失败
        """
        if not agent_id:
            raise ValueError("Agent ID cannot be empty")
        
        logger.debug(
            f"[RELATIONSHIP] Getting Agent service relations: agent_id={agent_id}"
        )
        
        # 获取关系
        relation_data = await self._cache_layer.get_relation(
            "agent_services",
            agent_id
        )
        
        if relation_data is None:
            logger.debug(
                f"[RELATIONSHIP] Agent relation does not exist: agent_id={agent_id}"
            )
            return []
        
        # 解析关系
        relation = AgentServiceRelation.from_dict(relation_data)
        
        # 转换为字典列表
        services = [service.to_dict() for service in relation.services]
        
        logger.debug(
            f"[RELATIONSHIP] Retrieved {len(services)} service relations: "
            f"agent_id={agent_id}"
        )
        
        return services
    
    # ==================== Service-Tool 关系管理 ====================
    
    async def add_service_tool(
        self,
        service_global_name: str,
        service_original_name: str,
        source_agent: str,
        tool_global_name: str,
        tool_original_name: str
    ) -> None:
        """
        添加 Service-Tool 关系
        
        Args:
            service_global_name: 服务全局名称
            service_original_name: 服务原始名称
            source_agent: 来源 Agent
            tool_global_name: 工具全局名称
            tool_original_name: 工具原始名称
            
        Raises:
            ValueError: 如果参数无效
            KeyError: 如果工具实体不存在
            RuntimeError: 如果添加失败
        """
        if not service_global_name:
            raise ValueError("Service global name cannot be empty")
        if not service_original_name:
            raise ValueError("Service original name cannot be empty")
        if not source_agent:
            raise ValueError("Source Agent cannot be empty")
        if not tool_global_name:
            raise ValueError("Tool global name cannot be empty")
        if not tool_original_name:
            raise ValueError("Tool original name cannot be empty")
        
        # 注意：不在这里验证工具实体存在性
        # 原因：
        # 1. 在 cache_manager._create_tool_entities_and_relations 中，
        #    create_tool 和 add_service_tool 是顺序调用的
        # 2. Redis 写入后可能存在短暂的读取延迟
        # 3. 关系层和实体层是独立的，关系层不应该依赖实体层的即时可读性
        # 4. 调用方负责确保工具实体已创建
        
        logger.debug(
            f"[RELATIONSHIP] Adding Service-Tool relation: "
            f"service_global_name={service_global_name}, "
            f"tool_global_name={tool_global_name}"
        )
        
        # 获取现有关系
        relation_data = await self._cache_layer.get_relation(
            "service_tools",
            service_global_name
        )
        
        if relation_data is None:
            # 创建新关系
            relation = ServiceToolRelation(
                service_global_name=service_global_name,
                service_original_name=service_original_name,
                source_agent=source_agent,
                tools=[]
            )
        else:
            # 解析现有关系
            relation = ServiceToolRelation.from_dict(relation_data)
        
        # 检查工具是否已存在（基于全局名称判断）
        for i, tool in enumerate(relation.tools):
            if tool.tool_global_name == tool_global_name:
                # 全局名称相同，认为是同一个关系，更新配置
                relation.tools[i] = ToolRelationItem(
                    tool_global_name=tool_global_name,
                    tool_original_name=tool_original_name
                )
                
                await self._cache_layer.put_relation(
                    "service_tools",
                    service_global_name,
                    relation.to_dict()
                )
                
                logger.info(
                    f"[RELATIONSHIP] Updated Service-Tool relation: "
                    f"service_global_name={service_global_name}, "
                    f"tool_global_name={tool_global_name}"
                )
                return
        
        # 添加新工具
        new_tool = ToolRelationItem(
            tool_global_name=tool_global_name,
            tool_original_name=tool_original_name
        )
        relation.tools.append(new_tool)
        
        # 保存关系
        await self._cache_layer.put_relation(
            "service_tools",
            service_global_name,
            relation.to_dict()
        )
        
        logger.info(
            f"[RELATIONSHIP] Successfully added Service-Tool relation: "
            f"service_global_name={service_global_name}, "
            f"tool_global_name={tool_global_name}"
        )
    
    async def remove_service_tool(
        self,
        service_global_name: str,
        tool_global_name: str
    ) -> None:
        """
        移除 Service-Tool 关系
        
        Args:
            service_global_name: 服务全局名称
            tool_global_name: 工具全局名称
            
        Raises:
            ValueError: 如果参数无效
            KeyError: 如果关系不存在
            RuntimeError: 如果移除失败
        """
        if not service_global_name:
            raise ValueError("Service global name cannot be empty")
        if not tool_global_name:
            raise ValueError("Tool global name cannot be empty")
        
        logger.debug(
            f"[RELATIONSHIP] Removing Service-Tool relation: "
            f"service_global_name={service_global_name}, "
            f"tool_global_name={tool_global_name}"
        )
        
        # 获取现有关系
        relation_data = await self._cache_layer.get_relation(
            "service_tools",
            service_global_name
        )
        
        if relation_data is None:
            raise KeyError(
                f"Service relation does not exist: service_global_name={service_global_name}"
            )
        
        # 解析关系
        relation = ServiceToolRelation.from_dict(relation_data)
        
        # 查找并移除工具
        original_count = len(relation.tools)
        relation.tools = [
            tool for tool in relation.tools
            if tool.tool_global_name != tool_global_name
        ]
        
        if len(relation.tools) == original_count:
            raise KeyError(
                f"Tool does not exist in service relation: "
                f"service_global_name={service_global_name}, "
                f"tool_global_name={tool_global_name}"
            )
        
        # 保存更新后的关系
        if len(relation.tools) == 0:
            # 如果没有工具了，删除整个关系
            await self._cache_layer.delete_relation(
                "service_tools",
                service_global_name
            )
            logger.info(
                f"[RELATIONSHIP] Deleted empty service relation: "
                f"service_global_name={service_global_name}"
            )
        else:
            # 保存更新后的关系
            await self._cache_layer.put_relation(
                "service_tools",
                service_global_name,
                relation.to_dict()
            )
            logger.info(
                f"[RELATIONSHIP] Successfully removed Service-Tool relation: "
                f"service_global_name={service_global_name}, "
                f"tool_global_name={tool_global_name}"
            )
    
    async def get_service_tools(
        self,
        service_global_name: str
    ) -> List[Dict[str, Any]]:
        """
        获取服务的所有工具关系
        
        Args:
            service_global_name: 服务全局名称
            
        Returns:
            工具关系列表，如果不存在返回空列表
            
        Raises:
            ValueError: 如果参数无效
            RuntimeError: 如果获取失败
        """
        if not service_global_name:
            raise ValueError("Service global name cannot be empty")
        
        # 每个服务只在首次读取关系时打印一条调试日志，避免在高频轮询中刷屏
        if service_global_name not in self._service_tools_logged_once:
            logger.debug(
                f"[RELATIONSHIP] Getting service tool relations: "
                f"service_global_name={service_global_name}"
            )
            self._service_tools_logged_once.add(service_global_name)
        
        # 获取关系
        relation_data = await self._cache_layer.get_relation(
            "service_tools",
            service_global_name
        )
        
        if relation_data is None:
            # 首次缺失使用 INFO 提示，之后按次数聚合为 DEBUG，避免刷屏
            missing_count = self._service_relation_missing_counts.get(service_global_name, 0) + 1
            self._service_relation_missing_counts[service_global_name] = missing_count
            if missing_count == 1:
                logger.info(
                    f"[RELATIONSHIP] Service relation does not exist: "
                    f"service_global_name={service_global_name}"
                )
            elif missing_count % 50 == 0:
                logger.debug(
                    f"[RELATIONSHIP] Service relation still missing: "
                    f"service_global_name={service_global_name}, missing_count={missing_count}"
                )
            return []
        
        # 解析关系
        relation = ServiceToolRelation.from_dict(relation_data)
        
        # 转换为字典列表
        tools = [tool.to_dict() for tool in relation.tools]
        
        # 仅在工具数量发生变化时输出调试日志，避免重复刷日志
        current_count = len(tools)
        previous_count = self._last_service_tool_counts.get(service_global_name)
        if previous_count != current_count:
            logger.debug(
                f"[RELATIONSHIP] Retrieved {current_count} tool relations: "
                f"service_global_name={service_global_name}"
            )
            self._last_service_tool_counts[service_global_name] = current_count
        
        return tools
    
    # ==================== 级联删除操作 ====================
    
    async def remove_service_cascade(
        self,
        agent_id: str,
        service_global_name: str
    ) -> None:
        """
        级联删除服务相关的所有关系
        
        删除顺序：
        1. 移除 Agent-Service 关系
        2. 删除 Service-Tool 关系
        
        Args:
            agent_id: Agent ID
            service_global_name: 服务全局名称
            
        Raises:
            ValueError: 如果参数无效
            RuntimeError: 如果删除失败
        """
        if not agent_id:
            raise ValueError("Agent ID cannot be empty")
        if not service_global_name:
            raise ValueError("Service global name cannot be empty")
        
        logger.info(
            f"[RELATIONSHIP] Cascading delete service relations: agent_id={agent_id}, "
            f"service_global_name={service_global_name}"
        )
        
        # 1. 移除 Agent-Service 关系
        try:
            await self.remove_agent_service(agent_id, service_global_name)
        except KeyError as e:
            logger.warning(
                f"[RELATIONSHIP] Agent-Service relation does not exist, skipping: {e}"
            )
        
        # 2. 删除 Service-Tool 关系
        try:
            await self._cache_layer.delete_relation(
                "service_tools",
                service_global_name
            )
            logger.info(
                f"[RELATIONSHIP] Deleted Service-Tool relation: "
                f"service_global_name={service_global_name}"
            )
        except Exception as e:
            logger.warning(
                f"[RELATIONSHIP] Failed to delete Service-Tool relation: {e}"
            )
        
        logger.info(
            f"[RELATIONSHIP] Cascading delete completed: service_global_name={service_global_name}"
        )



"""SamplingTool for use during LLM sampling requests."""

from __future__ import annotations

import inspect
from collections.abc import Callable
from typing import Any

from mcp.types import Tool as SDKTool
from pydantic import ConfigDict

from mcpstore.mcp.tools.function_parsing import ParsedFunction
from mcpstore.mcp.utilities.types import MCPStoreBaseModel


class SamplingTool(MCPStoreBaseModel):
    """A tool that can be used during LLM sampling.

    SamplingTools bundle a tool's schema (name, description, parameters) with
    an executor function, enabling servers to execute agentic workflows where
    the LLM can request tool calls during sampling.

    In most cases, pass functions directly to ctx.sample():

        def search(query: str) -> str:
            '''Search the web.'''
            return web_search(query)

        result = await context.sample(
            messages="Find info about Python",
            tools=[search],  # Plain functions work directly
        )

    Create a SamplingTool explicitly when you need custom name/description:

        tool = SamplingTool.from_function(search, name="web_search")
    """

    name: str
    description: str | None = None
    parameters: dict[str, Any]
    fn: Callable[..., Any]

    model_config = ConfigDict(arbitrary_types_allowed=True)

    async def run(self, arguments: dict[str, Any] | None = None) -> Any:
        """Execute the tool with the given arguments.

        Args:
            arguments: Dictionary of arguments to pass to the tool function.

        Returns:
            The result of executing the tool function.
        """
        if arguments is None:
            arguments = {}

        result = self.fn(**arguments)
        if inspect.isawaitable(result):
            result = await result
        return result

    def _to_sdk_tool(self) -> SDKTool:
        """Convert to an mcp.types.Tool for SDK compatibility.

        This is used internally when passing tools to the MCP SDK's
        create_message() method.
        """
        return SDKTool(
            name=self.name,
            description=self.description,
            inputSchema=self.parameters,
        )

    @classmethod
    def from_function(
        cls,
        fn: Callable[..., Any],
        *,
        name: str | None = None,
        description: str | None = None,
    ) -> SamplingTool:
        """Create a SamplingTool from a function.

        The function's signature is analyzed to generate a JSON schema for
        the tool's parameters. Type hints are used to determine parameter types.

        Args:
            fn: The function to create a tool from.
            name: Optional name override. Defaults to the function's name.
            description: Optional description override. Defaults to the function's docstring.

        Returns:
            A SamplingTool wrapping the function.

        Raises:
            ValueError: If the function is a lambda without a name override.
        """
        parsed = ParsedFunction.from_function(fn, validate=True)

        if name is None and parsed.name == "<lambda>":
            raise ValueError("You must provide a name for lambda functions")

        return cls(
            name=name or parsed.name,
            description=description or parsed.description,
            parameters=parsed.input_schema,
            fn=parsed.fn,
        )



"""Transport-related methods for MCPStore Server."""

from __future__ import annotations

from collections.abc import Awaitable, Callable
from functools import partial
from typing import TYPE_CHECKING, Any, Literal

import anyio
import uvicorn
from mcp.server.lowlevel.server import NotificationOptions
from mcp.server.stdio import stdio_server
from starlette.middleware import Middleware as ASGIMiddleware
from starlette.requests import Request
from starlette.responses import Response
from starlette.routing import BaseRoute, Route

from mcpstore.mcp import settings
from mcpstore.mcp.server.event_store import EventStore
from mcpstore.mcp.server.http import (
    StarletteWithLifespan,
    create_sse_app,
    create_streamable_http_app,
)
from mcpstore.mcp.utilities.cli import log_server_banner
from mcpstore.mcp.utilities.logging import get_logger, temporary_log_level

if TYPE_CHECKING:
    from mcpstore.mcp.server.server import MCPStore, Transport

logger = get_logger(__name__)


class TransportMixin:
    """Mixin providing transport-related methods for MCPStore.

    Includes HTTP/stdio/SSE transport handling and custom HTTP routes.
    """

    async def run_async(
        self: MCPStore,
        transport: Transport | None = None,
        show_banner: bool | None = None,
        **transport_kwargs: Any,
    ) -> None:
        """Run the MCPStore server asynchronously.

        Args:
            transport: Transport protocol to use ("stdio", "http", "sse", or "streamable-http")
            show_banner: Whether to display the server banner. If None, uses the
                MCPSTORE_SHOW_SERVER_BANNER setting (default: True).
        """
        if show_banner is None:
            show_banner = settings.show_server_banner
        if transport is None:
            transport = "stdio"
        if transport not in {"stdio", "http", "sse", "streamable-http"}:
            raise ValueError(f"Unknown transport: {transport}")

        if transport == "stdio":
            await self.run_stdio_async(
                show_banner=show_banner,
                **transport_kwargs,
            )
        elif transport in {"http", "sse", "streamable-http"}:
            await self.run_http_async(
                transport=transport,
                show_banner=show_banner,
                **transport_kwargs,
            )
        else:
            raise ValueError(f"Unknown transport: {transport}")

    def run(
        self: MCPStore,
        transport: Transport | None = None,
        show_banner: bool | None = None,
        **transport_kwargs: Any,
    ) -> None:
        """Run the MCPStore server. Note this is a synchronous function.

        Args:
            transport: Transport protocol to use ("http", "stdio", "sse", or "streamable-http")
            show_banner: Whether to display the server banner. If None, uses the
                MCPSTORE_SHOW_SERVER_BANNER setting (default: True).
        """

        anyio.run(
            partial(
                self.run_async,
                transport,
                show_banner=show_banner,
                **transport_kwargs,
            )
        )

    def custom_route(
        self: MCPStore,
        path: str,
        methods: list[str],
        name: str | None = None,
        include_in_schema: bool = True,
    ) -> Callable[
        [Callable[[Request], Awaitable[Response]]],
        Callable[[Request], Awaitable[Response]],
    ]:
        """
        Decorator to register a custom HTTP route on the MCPStore server.

        Allows adding arbitrary HTTP endpoints outside the standard MCP protocol,
        which can be useful for OAuth callbacks, health checks, or admin APIs.
        The handler function must be an async function that accepts a Starlette
        Request and returns a Response.

        Args:
            path: URL path for the route (e.g., "/auth/callback")
            methods: List of HTTP methods to support (e.g., ["GET", "POST"])
            name: Optional name for the route (to reference this route with
                Starlette's reverse URL lookup feature)
            include_in_schema: Whether to include in OpenAPI schema, defaults to True

        Example:
            Register a custom HTTP route for a health check endpoint:
            ```python
            @server.custom_route("/health", methods=["GET"])
            async def health_check(request: Request) -> Response:
                return JSONResponse({"status": "ok"})
            ```
        """

        def decorator(
            fn: Callable[[Request], Awaitable[Response]],
        ) -> Callable[[Request], Awaitable[Response]]:
            self._additional_http_routes.append(
                Route(
                    path,
                    endpoint=fn,
                    methods=methods,
                    name=name,
                    include_in_schema=include_in_schema,
                )
            )
            return fn

        return decorator

    def _get_additional_http_routes(self: MCPStore) -> list[BaseRoute]:
        """Get all additional HTTP routes including from providers.

        Returns a list of all custom HTTP routes from this server and
        from all providers that have HTTP routes (e.g., MCPStoreProvider).

        Returns:
            List of Starlette BaseRoute objects
        """
        return list(self._additional_http_routes)

    async def run_stdio_async(
        self: MCPStore,
        show_banner: bool = True,
        log_level: str | None = None,
        stateless: bool = False,
    ) -> None:
        """Run the server using stdio transport.

        Args:
            show_banner: Whether to display the server banner
            log_level: Log level for the server
            stateless: Whether to run in stateless mode (no session initialization)
        """
        from mcpstore.mcp.server.context import reset_transport, set_transport

        # Display server banner
        if show_banner:
            log_server_banner(server=self)

        token = set_transport("stdio")
        try:
            with temporary_log_level(log_level):
                async with self._lifespan_manager():
                    async with stdio_server() as (read_stream, write_stream):
                        mode = " (stateless)" if stateless else ""
                        logger.info(
                            f"Starting MCP server {self.name!r} with transport 'stdio'{mode}"
                        )

                        await self._mcp_server.run(
                            read_stream,
                            write_stream,
                            self._mcp_server.create_initialization_options(
                                notification_options=NotificationOptions(
                                    tools_changed=True
                                ),
                            ),
                            stateless=stateless,
                        )
        finally:
            reset_transport(token)

    async def run_http_async(
        self: MCPStore,
        show_banner: bool = True,
        transport: Literal["http", "streamable-http", "sse"] = "http",
        host: str | None = None,
        port: int | None = None,
        log_level: str | None = None,
        path: str | None = None,
        uvicorn_config: dict[str, Any] | None = None,
        middleware: list[ASGIMiddleware] | None = None,
        json_response: bool | None = None,
        stateless_http: bool | None = None,
        stateless: bool | None = None,
    ) -> None:
        """Run the server using HTTP transport.

        Args:
            transport: Transport protocol to use - "http" (default), "streamable-http", or "sse"
            host: Host address to bind to (defaults to settings.host)
            port: Port to bind to (defaults to settings.port)
            log_level: Log level for the server (defaults to settings.log_level)
            path: Path for the endpoint (defaults to settings.streamable_http_path or settings.sse_path)
            uvicorn_config: Additional configuration for the Uvicorn server
            middleware: A list of middleware to apply to the app
            json_response: Whether to use JSON response format (defaults to settings.json_response)
            stateless_http: Whether to use stateless HTTP (defaults to settings.stateless_http)
            stateless: Alias for stateless_http for CLI consistency
        """
        # Allow stateless as alias for stateless_http
        if stateless is not None and stateless_http is None:
            stateless_http = stateless

        # Resolve from settings/env var if not explicitly set
        if stateless_http is None:
            stateless_http = self._deprecated_settings.stateless_http

        # SSE doesn't support stateless mode
        if stateless_http and transport == "sse":
            raise ValueError("SSE transport does not support stateless mode")

        host = host or self._deprecated_settings.host
        port = port or self._deprecated_settings.port
        default_log_level_to_use = (
            log_level or self._deprecated_settings.log_level
        ).lower()

        app = self.http_app(
            path=path,
            transport=transport,
            middleware=middleware,
            json_response=json_response,
            stateless_http=stateless_http,
        )

        # Display server banner
        if show_banner:
            log_server_banner(server=self)
        uvicorn_config_from_user = uvicorn_config or {}

        config_kwargs: dict[str, Any] = {
            "timeout_graceful_shutdown": 0,
            "lifespan": "on",
            "ws": "websockets-sansio",
        }
        config_kwargs.update(uvicorn_config_from_user)

        if "log_config" not in config_kwargs and "log_level" not in config_kwargs:
            config_kwargs["log_level"] = default_log_level_to_use

        with temporary_log_level(log_level):
            async with self._lifespan_manager():
                config = uvicorn.Config(app, host=host, port=port, **config_kwargs)
                server = uvicorn.Server(config)
                path = getattr(app.state, "path", "").lstrip("/")
                mode = " (stateless)" if stateless_http else ""
                logger.info(
                    f"Starting MCP server {self.name!r} with transport {transport!r}{mode} on http://{host}:{port}/{path}"
                )

                await server.serve()

    def http_app(
        self: MCPStore,
        path: str | None = None,
        middleware: list[ASGIMiddleware] | None = None,
        json_response: bool | None = None,
        stateless_http: bool | None = None,
        transport: Literal["http", "streamable-http", "sse"] = "http",
        event_store: EventStore | None = None,
        retry_interval: int | None = None,
    ) -> StarletteWithLifespan:
        """Create a Starlette app using the specified HTTP transport.

        Args:
            path: The path for the HTTP endpoint
            middleware: A list of middleware to apply to the app
            json_response: Whether to use JSON response format
            stateless_http: Whether to use stateless mode (new transport per request)
            transport: Transport protocol to use - "http", "streamable-http", or "sse"
            event_store: Optional event store for SSE polling/resumability. When set,
                enables clients to reconnect and resume receiving events after
                server-initiated disconnections. Only used with streamable-http transport.
            retry_interval: Optional retry interval in milliseconds for SSE polling.
                Controls how quickly clients should reconnect after server-initiated
                disconnections. Requires event_store to be set. Only used with
                streamable-http transport.

        Returns:
            A Starlette application configured with the specified transport
        """

        if transport in ("streamable-http", "http"):
            return create_streamable_http_app(
                server=self,
                streamable_http_path=path
                or self._deprecated_settings.streamable_http_path,
                event_store=event_store,
                retry_interval=retry_interval,
                auth=self.auth,
                json_response=(
                    json_response
                    if json_response is not None
                    else self._deprecated_settings.json_response
                ),
                stateless_http=(
                    stateless_http
                    if stateless_http is not None
                    else self._deprecated_settings.stateless_http
                ),
                debug=self._deprecated_settings.debug,
                middleware=middleware,
            )
        elif transport == "sse":
            return create_sse_app(
                server=self,
                message_path=self._deprecated_settings.message_path,
                sse_path=path or self._deprecated_settings.sse_path,
                auth=self.auth,
                debug=self._deprecated_settings.debug,
                middleware=middleware,
            )
        else:
            raise ValueError(f"Unknown transport: {transport}")



"""
依赖注入容器 - 管理所有组件的创建和依赖关系

职责:
1. 创建和管理所有组件的生命周期
2. 处理组件之间的依赖关系
3. 提供统一的访问接口
"""

import logging
from typing import TYPE_CHECKING

from mcpstore.core.application.service_application_service import ServiceApplicationService
from mcpstore.core.domain.cache_manager import CacheManager
from mcpstore.core.domain.connection_manager import ConnectionManager
from mcpstore.core.domain.health_monitor import HealthMonitor
from mcpstore.core.domain.lifecycle_manager import LifecycleManager
from mcpstore.core.domain.persistence_manager import PersistenceManager
from mcpstore.core.domain.reconnection_scheduler import ReconnectionScheduler
from mcpstore.core.events.event_bus import EventBus

if TYPE_CHECKING:
    from mcpstore.core.registry.core_registry import CoreRegistry
    from mcpstore.core.registry.agent_locks import AgentLocks
    from mcpstore.core.configuration.unified_config import UnifiedConfigManager
    from mcpstore.core.configuration.config_processor import ConfigProcessor
    from mcpstore.core.integration.local_service_adapter import LocalServiceManagerAdapter

logger = logging.getLogger(__name__)


class ServiceContainer:
    """
    服务容器 - 依赖注入容器
    
    负责创建和管理所有组件的生命周期
    """
    
    def __init__(
        self,
        registry: 'CoreRegistry',
        agent_locks: 'AgentLocks',
        config_manager: 'UnifiedConfigManager',
        config_processor: 'ConfigProcessor',
        local_service_manager: 'LocalServiceManagerAdapter',
        global_agent_store_id: str,
        enable_event_history: bool = False,
        event_store=None,
        event_syncer=None,
        health_enabled: bool = True,
        reconnection_enabled: bool = True,
        enable_event_log: bool = True,
        is_only_db: bool = False,
        event_lease_ttl: float = 30.0,
    ):
        self._registry = registry
        self._agent_locks = agent_locks
        self._config_manager = config_manager
        self._config_processor = config_processor
        self._local_service_manager = local_service_manager
        self._global_agent_store_id = global_agent_store_id
        self._event_store = event_store
        self._event_syncer = event_syncer
        self._health_enabled = health_enabled
        self._reconnection_enabled = reconnection_enabled
        self._enable_event_log = enable_event_log
        self._is_only_db = is_only_db
        self._event_lease_ttl = event_lease_ttl
        
        # 创建事件总线（核心）
        # 事件总线：启用可选的 handler 超时（安全兜底）
        self._event_bus = EventBus(enable_history=enable_event_history, handler_timeout=None)
        
        # 创建领域服务
        self._cache_manager = CacheManager(
            event_bus=self._event_bus,
            registry=self._registry,
            agent_locks=self._agent_locks
        )

        # 获取生命周期配置（自动处理异步上下文和fallback）
        from mcpstore.config.toml_config import get_lifecycle_config_with_defaults
        lifecycle_config = get_lifecycle_config_with_defaults()

        # 获取HTTP超时配置
        from mcpstore.config.config_defaults import StandaloneConfigDefaults
        http_timeout_seconds = float(StandaloneConfigDefaults().http_timeout_seconds)
        logger.debug(f"[CONTAINER] HTTP timeout configured: {http_timeout_seconds} seconds")

        self._lifecycle_manager = LifecycleManager(
            event_bus=self._event_bus,
            registry=self._registry,
            lifecycle_config=lifecycle_config,
            agent_locks=self._agent_locks
        )

        self._connection_manager = ConnectionManager(
            event_bus=self._event_bus,
            registry=self._registry,
            config_processor=self._config_processor,
            local_service_manager=self._local_service_manager,
            http_timeout_seconds=http_timeout_seconds
        )

        self._persistence_manager = PersistenceManager(
            event_bus=self._event_bus,
            config_manager=self._config_manager,
            enable_file_persistence=False,
        )

        self._health_monitor = HealthMonitor(
            event_bus=self._event_bus,
            registry=self._registry,
            lifecycle_config=lifecycle_config,
            global_agent_store_id=self._global_agent_store_id,
            event_store=self._event_store,
            enable_event_log=self._enable_event_log,
            is_only_db=self._is_only_db,
            lease_ttl=self._event_lease_ttl,
        )

        # 创建重连调度器（使用相同的生命周期配置）
        self._reconnection_scheduler = ReconnectionScheduler(
            event_bus=self._event_bus,
            registry=self._registry,
            lifecycle_config=lifecycle_config,
            scan_interval=1.0,  # 扫描间隔固定1秒
            event_store=self._event_store,
            enable_event_log=self._enable_event_log,
            is_only_db=self._is_only_db,
            lease_ttl=self._event_lease_ttl,
        )

        # 创建应用服务
        self._service_app_service = ServiceApplicationService(
            event_bus=self._event_bus,
            registry=self._registry,
            lifecycle_manager=self._lifecycle_manager,
            global_agent_store_id=self._global_agent_store_id,
            event_store=self._event_store,
            enable_event_log=self._enable_event_log,
            is_only_db=self._is_only_db,
        )

        # 事件请求监听（控制平面执行）
        from mcpstore.core.events.service_events import ServiceRestartRequested, ServiceResetRequested

        async def _on_restart_requested(event):
            try:
                await self._service_app_service.restart_service(
                    service_name=event.service_name,
                    agent_id=event.agent_id,
                    wait_timeout=0.0,
                    from_event=True,
                    source=event.source,
                )
            except Exception as e:
                logger.error(f"[EVENT] ServiceRestartRequested handling failed: {e}", exc_info=True)

        self._event_bus.subscribe(ServiceRestartRequested, _on_restart_requested, priority=90)

        async def _on_reset_requested(event):
            try:
                await self._service_app_service.reset_service(
                    agent_id=event.agent_id,
                    service_name=event.service_name,
                    wait_timeout=0.0,
                    from_event=True,
                    source=event.source,
                )
            except Exception as e:
                logger.error(f"[EVENT] ServiceResetRequested handling failed: {e}", exc_info=True)

        self._event_bus.subscribe(ServiceResetRequested, _on_reset_requested, priority=90)

        # 事件诊断订阅：记录就绪/持久化阶段，便于调试与监控
        async def _on_service_persisting(event):
            logger.info(
                "[EVENT] ServicePersisting: agent=%s service=%s stage=%s tools=%s",
                event.agent_id, event.service_name, getattr(event, "stage", "cache"), getattr(event, "tool_count", 0)
            )

        async def _on_service_persisted(event):
            logger.info(
                "[EVENT] ServicePersisted: agent=%s service=%s stage=%s tools=%s",
                event.agent_id, event.service_name, getattr(event, "stage", "config"), getattr(event, "tool_count", 0)
            )

        async def _on_service_ready(event):
            logger.info(
                "[EVENT] ServiceReady: agent=%s service=%s health=%s tools=%s",
                event.agent_id, event.service_name, getattr(event, "health_status", ""), getattr(event, "tool_count", 0)
            )

        async def _on_tool_sync(event):
            logger.info(
                "[EVENT] ToolSync: agent=%s service=%s total=%s phase=%s",
                event.agent_id, event.service_name, getattr(event, "total_tools", 0),
                event.__class__.__name__
            )

        from mcpstore.core.events.service_events import (
            ServicePersisting,
            ServiceReady,
            ToolSyncStarted,
            ToolSyncCompleted,
        )
        self._event_bus.subscribe(ServicePersisting, _on_service_persisting, priority=1)
        # ServicePersisted 已停用文件持久化，不再订阅
        self._event_bus.subscribe(ServiceReady, _on_service_ready, priority=1)
        self._event_bus.subscribe(ToolSyncStarted, _on_tool_sync, priority=1)
        self._event_bus.subscribe(ToolSyncCompleted, _on_tool_sync, priority=1)

        logger.info("ServiceContainer initialized with all components (including health monitor and reconnection scheduler)")
    
    @property
    def event_bus(self) -> EventBus:
        """获取事件总线"""
        return self._event_bus
    
    @property
    def service_application_service(self) -> ServiceApplicationService:
        """获取服务应用服务"""
        return self._service_app_service
    
    @property
    def cache_manager(self) -> CacheManager:
        """获取缓存管理器"""
        return self._cache_manager
    
    @property
    def lifecycle_manager(self) -> LifecycleManager:
        """获取生命周期管理器"""
        return self._lifecycle_manager
    
    @property
    def connection_manager(self) -> ConnectionManager:
        """获取连接管理器"""
        return self._connection_manager
    
    @property
    def persistence_manager(self) -> PersistenceManager:
        """获取持久化管理器"""
        return self._persistence_manager

    @property
    def health_monitor(self) -> HealthMonitor:
        """获取健康监控管理器"""
        return self._health_monitor

    @property
    def reconnection_scheduler(self) -> ReconnectionScheduler:
        """获取重连调度器"""
        return self._reconnection_scheduler

    async def start(self):
        """启动所有需要后台运行的组件"""
        logger.info("Starting ServiceContainer components...")

        if self._event_syncer:
            try:
                await self._event_syncer.start()
                logger.info("EventSyncer started")
            except Exception as sync_err:
                logger.error(f"EventSyncer start failed: {sync_err}", exc_info=True)

        # 启动健康监控
        if self._health_enabled:
            await self._health_monitor.start()
        else:
            logger.info("HealthMonitor disabled (mode=only_db)")

        # 启动重连调度器
        if self._reconnection_enabled:
            await self._reconnection_scheduler.start()
        else:
            logger.info("ReconnectionScheduler disabled (mode=only_db)")

        logger.info("ServiceContainer components started")

    async def stop(self):
        """停止所有组件"""
        logger.info("Stopping ServiceContainer components...")

        if self._event_syncer:
            try:
                await self._event_syncer.stop()
            except Exception:
                pass

        # 停止健康监控
        if self._health_enabled:
            await self._health_monitor.stop()

        # 停止重连调度器
        if self._reconnection_enabled:
            await self._reconnection_scheduler.stop()

        logger.info("ServiceContainer components stopped")

    def attach_event_syncer(self, event_syncer):
        """在容器创建后注入事件同步器。"""
        self._event_syncer = event_syncer
        try:
            # 将 event_syncer 透传给应用服务，用于写后同步消费
            self._service_app_service._event_syncer = event_syncer
        except Exception:
            pass



"""
Async Orchestrated Bridge (AOB)

为同步 API 提供统一的异步执行桥梁。
"""

from .async_orchestrated_bridge import (
    AsyncOrchestratedBridge,
    get_async_bridge,
    close_async_bridge,
)
from .unified_executor import (
    BridgeExecutionStats,
    UnifiedBridgeExecutor,
    get_bridge_executor,
    get_bridge_execution_stats,
    reset_bridge_execution_stats,
)

__all__ = [
    "AsyncOrchestratedBridge",
    "get_async_bridge",
    "close_async_bridge",
    "BridgeExecutionStats",
    "UnifiedBridgeExecutor",
    "get_bridge_executor",
    "get_bridge_execution_stats",
    "reset_bridge_execution_stats",
]



#!/usr/bin/env python3
"""
MCPStore Standalone Configuration System
Works completely independent of environment variables, through default parameters and initialization configuration
"""

import logging
from copy import deepcopy
from dataclasses import dataclass, field
from typing import Dict, Any, Optional, Union

from ..registry.schema_manager import get_schema_manager
from ...config.config_defaults import StandaloneConfigDefaults

logger = logging.getLogger(__name__)

_standalone_defaults = StandaloneConfigDefaults()

@dataclass
class StandaloneConfig:
    """Standalone configuration class - does not depend on any environment variables"""

    # === Core configuration ===
    heartbeat_interval_seconds: int = int(_standalone_defaults.heartbeat_interval_seconds)
    http_timeout_seconds: int = int(_standalone_defaults.http_timeout_seconds)
    reconnection_interval_seconds: int = int(_standalone_defaults.reconnection_interval_seconds)
    cleanup_interval_seconds: int = int(_standalone_defaults.cleanup_interval_seconds)
    
    # === Network configuration ===
    streamable_http_endpoint: str = "/mcp"
    default_transport: str = _standalone_defaults.default_transport
    
    # === File path configuration ===
    config_dir: Optional[str] = None  # If None, use in-memory configuration
    mcp_config_file: Optional[str] = None
    # Single data source architecture: only support unified config
    
    # === Service configuration ===
    known_services: Dict[str, Dict[str, Any]] = field(default_factory=lambda: {})
    
    # === Environment configuration removed ===
    # Environment variable handling is now completely handled by MCPStore, no longer need these configurations

    # === Logging configuration ===
    log_level: str = _standalone_defaults.log_level
    log_format: str = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    enable_debug: bool = _standalone_defaults.enable_debug

class StandaloneConfigManager:
    """Standalone configuration manager - completely independent of environment variables"""
    
    def __init__(self, config: Optional[StandaloneConfig] = None):
        """
        Initialize standalone configuration manager

        Args:
            config: Custom configuration, if None use default configuration
        """
        self.config = config or StandaloneConfig()
        self._runtime_config: Dict[str, Any] = {}
        self._service_configs: Dict[str, Dict[str, Any]] = {}
        
        # Initialize default configuration
        self._initialize_default_configs()
        
        logger.info("StandaloneConfigManager initialized without environment dependencies")
    
    def _initialize_default_configs(self):
        """Initialize default configuration"""
        # Set up runtime configuration
        self._runtime_config = {
            "timing": {
                "heartbeat_interval_seconds": self.config.heartbeat_interval_seconds,
                "http_timeout_seconds": self.config.http_timeout_seconds,
                "reconnection_interval_seconds": self.config.reconnection_interval_seconds,
                "cleanup_interval_seconds": self.config.cleanup_interval_seconds
            },
            "network": {
                "streamable_http_endpoint": self.config.streamable_http_endpoint,
                "default_transport": self.config.default_transport
            },
            "environment": {
                "note": "Environment configuration removed - now handled by MCPStore"
            }
        }

        # 使用Schema管理器初始化已知服务配置
        schema_manager = get_schema_manager()
        self._service_configs = {
            "mcpstore-wiki": schema_manager.get_known_service_config("mcpstore-wiki"),
            "howtocook": schema_manager.get_known_service_config("howtocook")
        }
        # 合并用户自定义的服务配置
        self._service_configs.update(deepcopy(self.config.known_services))
    
    def get_timing_config(self) -> Dict[str, int]:
        """获取时间配置"""
        return self._runtime_config["timing"]
    
    def get_network_config(self) -> Dict[str, str]:
        """获取网络配置"""
        return self._runtime_config["network"]
    
    def get_environment_config(self) -> Dict[str, Any]:
        """获取环境配置"""
        return self._runtime_config["environment"]
    
    def get_service_config(self, service_name: str) -> Optional[Dict[str, Any]]:
        """获取服务配置"""
        return self._service_configs.get(service_name)
    
    def add_service_config(self, service_name: str, config: Dict[str, Any]):
        """添加服务配置"""
        self._service_configs[service_name] = deepcopy(config)
        logger.info(f"Added service config for: {service_name}")
    
    def get_all_service_configs(self) -> Dict[str, Dict[str, Any]]:
        """获取所有服务配置"""
        return deepcopy(self._service_configs)
    
    def get_mcp_config(self) -> Dict[str, Any]:
        """获取MCP格式的配置"""
        return {
            "mcpServers": deepcopy(self._service_configs),
            "version": "1.0.0",
            "description": "MCPStore standalone configuration"
        }
    
    def update_config(self, **kwargs):
        """更新配置"""
        for key, value in kwargs.items():
            if hasattr(self.config, key):
                setattr(self.config, key, value)
                logger.info(f"Updated config: {key} = {value}")
        
        # 重新初始化配置
        self._initialize_default_configs()
    
    # get_isolated_environment方法已删除 - 环境变量处理现在完全由MCPStore处理
    
    def get_config_paths(self) -> Dict[str, Optional[str]]:
        """获取配置文件路径"""
        return {
            "config_dir": self.config.config_dir,
            "mcp_config_file": self.config.mcp_config_file
        }
    
    def is_file_based(self) -> bool:
        """检查是否使用文件配置"""
        return self.config.config_dir is not None or self.config.mcp_config_file is not None

class StandaloneConfigBuilder:
    """独立配置构建器 - 提供流畅的配置构建接口"""
    
    def __init__(self):
        self._config = StandaloneConfig()
    
    def with_timing(self, heartbeat: int = None, timeout: int = None, reconnection: int = None) -> 'StandaloneConfigBuilder':
        """设置时间配置"""
        if heartbeat is not None:
            self._config.heartbeat_interval_seconds = heartbeat
        if timeout is not None:
            self._config.http_timeout_seconds = timeout
        if reconnection is not None:
            self._config.reconnection_interval_seconds = reconnection
        return self
    
    def with_network(self, endpoint: str = None, transport: str = None) -> 'StandaloneConfigBuilder':
        """设置网络配置"""
        if endpoint is not None:
            self._config.streamable_http_endpoint = endpoint
        if transport is not None:
            self._config.default_transport = transport
        return self
    
    def with_files(self, config_dir: str = None, mcp_file: str = None) -> 'StandaloneConfigBuilder':
        """设置文件配置"""
        if config_dir is not None:
            self._config.config_dir = config_dir
        if mcp_file is not None:
            self._config.mcp_config_file = mcp_file
        return self
    
    def with_service(self, name: str, config: Dict[str, Any]) -> 'StandaloneConfigBuilder':
        """添加服务配置"""
        self._config.known_services[name] = config
        return self
    
    
    def with_logging(self, level: str = None, debug: bool = None) -> 'StandaloneConfigBuilder':
        """设置日志配置"""
        if level is not None:
            self._config.log_level = level
        if debug is not None:
            self._config.enable_debug = debug
        return self
    
    def build(self) -> StandaloneConfig:
        """构建配置"""
        return deepcopy(self._config)

# === 预定义配置模板 ===

def create_minimal_config() -> StandaloneConfig:
    """创建最小配置 - 只包含基本功能"""
    return StandaloneConfigBuilder().build()

def create_development_config() -> StandaloneConfig:
    """创建开发配置 - 包含调试功能"""
    return (StandaloneConfigBuilder()
            .with_timing(heartbeat=30, timeout=10, reconnection=60)
            .with_logging(level="DEBUG", debug=True)
            .build())

# Removed preset configurations - MCPStore is just a tool, users decide their own configuration

# === 全局配置实例 ===
_global_config_manager: Optional[StandaloneConfigManager] = None

def get_global_config() -> StandaloneConfigManager:
    """获取全局配置管理器"""
    global _global_config_manager
    if _global_config_manager is None:
        _global_config_manager = StandaloneConfigManager()
    return _global_config_manager

def set_global_config(config: Union[StandaloneConfig, StandaloneConfigManager]):
    """设置全局配置"""
    global _global_config_manager
    if isinstance(config, StandaloneConfig):
        _global_config_manager = StandaloneConfigManager(config)
    else:
        _global_config_manager = config
    logger.info("Global standalone config updated")

def reset_global_config():
    """重置全局配置"""
    global _global_config_manager
    _global_config_manager = None
    logger.info("Global standalone config reset")




"""事件日志模块。"""

from mcpstore.core.eventlog.event_models import EventRecord  # noqa: F401
from mcpstore.core.eventlog.event_store import EventStore  # noqa: F401
from mcpstore.core.eventlog.event_syncer import EventSyncer  # noqa: F401



"""
MCPStore Service Management Module
服务管理相关操作的实现
"""

import asyncio
import datetime
import logging
import time
from typing import Dict, List, Optional, Any, Union, Tuple

from mcpstore.core.models.service import ServiceConnectionState
from .types import ContextType

logger = logging.getLogger(__name__)


class UpdateServiceAuthHelper:
    """更新服务认证助手 - 明确的服务名，避免状态混乱
    
    Note: 这是一个内部助手类，为了符合 async-only 约束，
    所有方法都改为 async，外部调用者需要 await。
    """

    def __init__(self, context: 'MCPStoreContext', service_name: str, config: Dict[str, Any] = None):
        self._context = context
        self._service_name = service_name  # [CONFIG] Clear service name to avoid confusion
        self._config = config.copy() if config else {}

    async def bearer_auth(self, auth: str) -> 'MCPStoreContext':
        """Update Bearer Token authentication for specified service (backward compatible)"""
        # Standardize to Authorization header
        if "headers" not in self._config:
            self._config["headers"] = {}
        self._config["headers"]["Authorization"] = f"Bearer {auth}"
        return await self._execute_update()

    async def token(self, token: str) -> 'MCPStoreContext':
        """Recommended: Set Bearer Token (equivalent to bearer_auth)"""
        if "headers" not in self._config:
            self._config["headers"] = {}
        self._config["headers"]["Authorization"] = f"Bearer {token}"
        return await self._execute_update()

    async def api_key(self, api_key: str) -> 'MCPStoreContext':
        """Recommended: Set API Key (standardized to X-API-Key)"""
        if "headers" not in self._config:
            self._config["headers"] = {}
        self._config["headers"]["X-API-Key"] = api_key
        return await self._execute_update()

    async def custom_headers(self, headers: Dict[str, str]) -> 'MCPStoreContext':
        """Update custom headers for specified service (explicit override)"""
        if "headers" not in self._config:
            self._config["headers"] = {}
        self._config["headers"].update(headers)
        return await self._execute_update()

    async def _execute_update(self) -> 'MCPStoreContext':
        """执行更新服务（内部 async-only）"""
        await self._context.update_service_async(self._service_name, self._config)
        return self._context


class ServiceManagementMixin:
    """服务管理混入类"""

    # [已删除] check_services 同步方法
    # 根据 "pykv 唯一真相数据源" 原则，请使用 check_services_async 异步方法

    async def check_services_async(self) -> dict:
        """
        异步健康检查，store/agent上下文自动判断
        - store上下文：聚合 global_agent_store 下所有 client_id 的服务健康状态
        - agent上下文：聚合 agent_id 下所有 client_id 的服务健康状态
        """
        if self._context_type.name == 'STORE':
            return await self._store.get_health_status()
        elif self._context_type.name == 'AGENT':
            return await self._store.get_health_status(self._agent_id, agent_mode=True)
        else:
            logger.error(f"[check_services] Unknown context type: {self._context_type}")
            return {}

    def get_service_info(self, name: str) -> Any:
        """
        获取服务详情（同步版本），支持 store/agent 上下文
        - store上下文：在 global_agent_store 下的所有 client 中查找服务
        - agent上下文：在指定 agent_id 下的所有 client 中查找服务

        [新架构] 避免_sync_helper.run_async，使用更安全的同步实现
        """
        try:
            if not name:
                return {}

            if self._context_type == ContextType.STORE:
                logger.debug(f"STORE mode - searching service in global_agent_store: {name}")
                agent_id = self._store.client_manager.global_agent_store_id
            else:
                logger.debug(f"AGENT mode - searching service in agent({self._agent_id}): {name}")
                agent_id = self._agent_id

            # 直接从缓存获取服务信息
            complete_info = self._store.registry.get_complete_service_info(agent_id, name)
            if not complete_info:
                logger.debug(f"Service {name} not found in agent {agent_id}")
                return {}

            # 构建返回信息
            return {
                "name": name,
                "client_id": complete_info.get("client_id"),
                "config": complete_info.get("config", {}),
                "state": complete_info.get("state", "disconnected"),
                "tool_count": complete_info.get("tool_count", 0),
                "agent_id": agent_id
            }

        except Exception as e:
            logger.error(f"[NEW_ARCH] get_service_info failed: {e}")
            return {
                "name": name,
                "error": str(e),
                "agent_id": getattr(self, '_agent_id', 'unknown')
            }

    # 别名：符合命名规范
    def service_info(self, name: str) -> Any:
        return self.get_service_info(name)

    async def get_service_info_async(self, name: str) -> Any:
        """
        获取服务详情（异步版本），支持 store/agent 上下文
        - store上下文：在 global_agent_store 下的所有 client 中查找服务
        - agent上下文：在指定 agent_id 下的所有 client 中查找服务（支持本地名称）
        """
        if not name:
            return {}

        if self._context_type == ContextType.STORE:
            logger.debug(f"STORE mode - searching service in global_agent_store: {name}")
            return await self._store.get_service_info(name)
        elif self._context_type == ContextType.AGENT:
            # Agent模式：将名称原样交给 Store 层处理，Store 负责本地名/全局名的鲁棒解析
            logger.debug(f"AGENT mode - searching service in agent({self._agent_id}): {name}")
            return await self._store.get_service_info(name, self._agent_id)
        else:
            logger.error(f"[get_service_info] Unknown context type: {self._context_type}")
            return {}

    async def service_info_async(self, name: str) -> Any:
        return await self.get_service_info_async(name)

    def update_service(self,
                      name: str,
                      config: Union[Dict[str, Any], None] = None,
                      # 🆕 与用户用法对齐
                      auth: Optional[str] = None,            # 兼容历史：等价于 token
                      token: Optional[str] = None,           # 推荐：Bearer Token
                      api_key: Optional[str] = None,         # 推荐：API Key
                      headers: Optional[Dict[str, str]] = None) -> Union['MCPStoreContext', 'UpdateServiceAuthHelper']:
        """
        更新服务配置，支持安全的链式认证与凭证轮换（合并更新，不会破坏原有关键字段）

        Args:
            name: 服务名称（明确指定，不会混乱）
            config: 新的服务配置（可选，按“补丁”合并语义处理）
            auth/token: Bearer token（两者等价；优先使用 token）
            api_key: API Key（统一标准化为 X-API-Key 头）
            headers: 自定义请求头（显式传入的键优先级最高）

        Returns:
            如果有配置或认证参数：立即执行更新，返回 MCPStoreContext
            如果什么都没有：返回 UpdateServiceAuthHelper 支持链式配置
        """

        if config is not None:
            # 有配置参数：立即执行更新（与认证参数合并，并采用“补丁合并”语义）
            if any([auth, token, api_key, headers]):
                final_config = self._apply_auth_to_update_config(config, auth, token, api_key, headers)
            else:
                final_config = config

            try:
                self._run_async_via_bridge(
                    self.update_service_async(name, final_config),
                    op_name="service_management.update_service"
                )
            except Exception as e:
                logger.error(f"[NEW_ARCH] update_service failed: {e}")
            return self
        else:
            # 没有配置参数：
            if any([auth, token, api_key, headers]):
                # 纯认证：立即执行（也走补丁合并语义）
                final_config = self._apply_auth_to_update_config({}, auth, token, api_key, headers)
                try:
                    self._run_async_via_bridge(
                        self.update_service_async(name, final_config),
                        op_name="service_management.update_service_auth"
                    )
                except Exception as e:
                    logger.error(f"[NEW_ARCH] update_service (auth) failed: {e}")
                return self
            else:
                # 什么都没有：返回助手用于链式调用
                return UpdateServiceAuthHelper(self, name, {})

    async def update_service_async(self, name: str, config: Dict[str, Any]) -> bool:
        """
        更新服务配置（异步版本）- 合并更新（不会破坏未提供的关键字段）

        Args:
            name: 服务名称
            config: 新的服务配置（作为补丁）

        Returns:
            bool: 更新是否成功
        """
        try:
            #  内部：简单的深度合并（仅对字典执行一层合并；headers 为字典则键级覆盖）
            def _deep_merge(base: Dict[str, Any], patch: Dict[str, Any]) -> Dict[str, Any]:
                result = dict(base or {})
                for k, v in (patch or {}).items():
                    if isinstance(v, dict) and isinstance(result.get(k), dict):
                        merged = dict(result.get(k) or {})
                        merged.update(v)
                        result[k] = merged
                    else:
                        result[k] = v
                return result

            if self._context_type == ContextType.STORE:
                # Store级别：使用原子更新，避免读改写竞态
                from mcpstore.core.configuration.config_write_service import ConfigWriteService
                from mcpstore.utils.perspective_resolver import PerspectiveResolver
                from mcpstore.core.exceptions import PerspectiveResolutionError

                try:
                    resolver = PerspectiveResolver()
                    name_res = resolver.normalize_service_name(
                        self._store.client_manager.global_agent_store_id,
                        name,
                        target="global",
                        strict=False,
                    )
                    config_key = name_res.global_name
                except Exception as e:
                    raise PerspectiveResolutionError(name, str(e)) from e

                cws = ConfigWriteService()
                def _mutator(cfg: Dict[str, Any]) -> Dict[str, Any]:
                    servers = dict(cfg.get("mcpServers", {}))
                    if config_key not in servers:
                        raise KeyError(f"Service {config_key} not found in store configuration")
                    existing = dict(servers.get(config_key) or {})
                    merged = _deep_merge(existing, config)
                    servers[config_key] = merged
                    cfg["mcpServers"] = servers
                    return cfg
                try:
                    success = cws.atomic_update(self._store.config.json_path, _mutator)
                except KeyError as e:
                    logger.error(str(e))
                    return False

                if success:
                    # 触发重新注册
                    if hasattr(self._store.orchestrator, 'sync_manager') and self._store.orchestrator.sync_manager:
                        await self._store.orchestrator.sync_manager.sync_global_agent_store_from_mcp_json()

                return success
            else:
                # Agent级别：与单一数据源模式对齐——直接更新 mcp.json 并触发同步
                from mcpstore.utils.perspective_resolver import PerspectiveResolver
                from mcpstore.core.exceptions import PerspectiveResolutionError

                try:
                    resolver = PerspectiveResolver()
                    name_res = resolver.normalize_service_name(
                        self._agent_id,
                        name,
                        target="global",
                        strict=False,
                    )
                    global_name = name_res.global_name
                except Exception as e:
                    raise PerspectiveResolutionError(name, str(e)) from e

                from mcpstore.core.configuration.config_write_service import ConfigWriteService
                cws = ConfigWriteService()
                def _mutator(cfg: Dict[str, Any]) -> Dict[str, Any]:
                    servers = dict(cfg.get("mcpServers", {}))
                    if global_name not in servers:
                        raise KeyError(f"Service {global_name} not found in store configuration (agent mode)")
                    existing = dict(servers.get(global_name) or {})
                    merged = _deep_merge(existing, config)
                    servers[global_name] = merged
                    cfg["mcpServers"] = servers
                    return cfg
                try:
                    success = cws.atomic_update(self._store.config.json_path, _mutator)
                except KeyError as e:
                    logger.error(str(e))
                    return False

                if success and hasattr(self._store.orchestrator, 'sync_manager') and self._store.orchestrator.sync_manager:
                    await self._store.orchestrator.sync_manager.sync_global_agent_store_from_mcp_json()

                # 更新缓存中的 metadata.service_config，确保一致性
                try:
                    # 从 pykv 异步获取元数据
                    global_agent = self._store.client_manager.global_agent_store_id
                    metadata = await self._store.registry._service_state_service.get_service_metadata_async(global_agent, global_name)
                    if metadata:
                        # 将变更合并到缓存元数据中
                        metadata.service_config = _deep_merge(metadata.service_config or {}, config)
                        await self._store.registry.set_service_metadata_async(global_agent, global_name, metadata)
                except Exception as e:
                    logger.error(f"Failed to update service metadata: {e}")
                    raise

                return success
        except Exception as e:
            logger.error(f"Failed to update service {name}: {e}")
            raise

    def patch_service(self, name: str, updates: Dict[str, Any]) -> bool:
        """
        增量更新服务配置（同步版本）- 推荐使用

        Args:
            name: 服务名称
            updates: 要更新的配置项

        Returns:
            bool: 更新是否成功
        """
        try:
            return self._run_async_via_bridge(
                self.patch_service_async(name, updates),
                op_name="service_management.patch_service"
            )
        except Exception as e:
            logger.error(f"[NEW_ARCH] patch_service failed: {e}")
            return False

    async def patch_service_async(self, name: str, updates: Dict[str, Any]) -> bool:
        """
        增量更新服务配置（异步版本）- 推荐使用

        Args:
            name: 服务名称
            updates: 要更新的配置项

        Returns:
            bool: 更新是否成功
        """
        try:
            if self._context_type == ContextType.STORE:
                # Store级别：使用原子增量更新
                from mcpstore.core.configuration.config_write_service import ConfigWriteService
                from mcpstore.utils.perspective_resolver import PerspectiveResolver
                from mcpstore.core.exceptions import PerspectiveResolutionError

                try:
                    resolver = PerspectiveResolver()
                    name_res = resolver.normalize_service_name(
                        self._store.client_manager.global_agent_store_id,
                        name,
                        target="global",
                        strict=False,
                    )
                    config_key = name_res.global_name
                except Exception as e:
                    raise PerspectiveResolutionError(name, str(e)) from e

                cws = ConfigWriteService()
                def _mutator(cfg: Dict[str, Any]) -> Dict[str, Any]:
                    servers = dict(cfg.get("mcpServers", {}))
                    if config_key not in servers:
                        raise KeyError(f"Service {config_key} not found in store configuration")
                    merged = dict(servers[config_key])
                    merged.update(updates)
                    servers[config_key] = merged
                    cfg["mcpServers"] = servers
                    return cfg
                try:
                    success = cws.atomic_update(self._store.config.json_path, _mutator)
                except KeyError as e:
                    logger.error(str(e))
                    return False

                if success:
                    # 触发重新注册
                    if hasattr(self._store.orchestrator, 'sync_manager') and self._store.orchestrator.sync_manager:
                        await self._store.orchestrator.sync_manager.sync_global_agent_store_from_mcp_json()

                return success
            else:
                # Agent级别：与单一数据源模式对齐——直接增量更新 mcp.json 并触发同步
                from mcpstore.utils.perspective_resolver import PerspectiveResolver
                from mcpstore.core.exceptions import PerspectiveResolutionError

                try:
                    resolver = PerspectiveResolver()
                    name_res = resolver.normalize_service_name(
                        self._agent_id,
                        name,
                        target="global",
                        strict=False,
                    )
                    global_name = name_res.global_name
                except Exception as e:
                    raise PerspectiveResolutionError(name, str(e)) from e

                from mcpstore.core.configuration.config_write_service import ConfigWriteService
                cws = ConfigWriteService()
                def _mutator(cfg: Dict[str, Any]) -> Dict[str, Any]:
                    servers = dict(cfg.get("mcpServers", {}))
                    if global_name not in servers:
                        raise KeyError(f"Service {global_name} not found in store configuration (agent mode)")
                    merged = dict(servers[global_name])
                    merged.update(updates)
                    servers[global_name] = merged
                    cfg["mcpServers"] = servers
                    return cfg
                try:
                    success = cws.atomic_update(self._store.config.json_path, _mutator)
                except KeyError as e:
                    logger.error(str(e))
                    return False

                if success and hasattr(self._store.orchestrator, 'sync_manager') and self._store.orchestrator.sync_manager:
                    await self._store.orchestrator.sync_manager.sync_global_agent_store_from_mcp_json()

                # 更新缓存中的 metadata.service_config，确保一致性
                try:
                    # 从 pykv 异步获取元数据
                    global_agent = self._store.client_manager.global_agent_store_id
                    metadata = await self._store.registry._service_state_service.get_service_metadata_async(global_agent, global_name)
                    if metadata:
                        metadata.service_config = dict(metadata.service_config or {})
                        metadata.service_config.update(updates)
                        await self._store.registry.set_service_metadata_async(global_agent, global_name, metadata)
                except Exception as e:
                    logger.error(f"Failed to update service metadata: {e}")
                    raise

                return success
        except Exception as e:
            logger.error(f"Failed to patch service {name}: {e}")
            raise

    def delete_service(self, name: str) -> bool:
        """
        删除服务（同步版本）

        Args:
            name: 服务名称

        Returns:
            bool: 删除是否成功
        """
        try:
            return self._run_async_via_bridge(
                self.delete_service_async(name),
                op_name="service_management.delete_service"
            )
        except Exception as e:
            logger.error(f"[NEW_ARCH] delete_service failed: {e}")
            return False

    async def delete_service_async(self, name: str) -> bool:
        """
        删除服务（异步版本，透明代理）

        Args:
            name: 服务名称（Agent 模式下使用本地名称）

        Returns:
            bool: 删除是否成功
        """
        try:
            if self._context_type == ContextType.STORE:
                # Store级别：删除服务并触发双向同步
                from mcpstore.utils.perspective_resolver import PerspectiveResolver
                from mcpstore.core.exceptions import PerspectiveResolutionError

                try:
                    resolver = PerspectiveResolver()
                    name_res = resolver.normalize_service_name(
                        self._store.client_manager.global_agent_store_id,
                        name,
                        target="global",
                        strict=False,
                    )
                    target_name = name_res.global_name
                except Exception as e:
                    raise PerspectiveResolutionError(name, str(e)) from e

                await self._delete_store_service_with_sync(target_name)
                return True
            else:
                # Agent级别：透明代理删除
                await self._delete_agent_service_with_sync(name)
                return True
        except Exception as e:
            logger.error(f"Failed to delete service {name}: {e}")
            raise

    def _normalize_agent_local_name(self, input_name: str) -> str:
        """
        将输入的服务标识归一化为当前 Agent 的本地服务名。

        支持三种输入：
        1) 纯本地名（默认）
        2) 全局名格式：<local>_byagent_<agent_id>
        3) 冒号分隔：<agent_id>:<local>

        若提供的 agent_id 与当前上下文不一致则抛出异常，避免误删。
        """
        if not input_name:
            raise ValueError("service name is required")

        # 统一委托给 PerspectiveResolver，避免重复实现视角转换
        try:
            from mcpstore.utils.perspective_resolver import PerspectiveResolver

            resolver = PerspectiveResolver()
            res = resolver.normalize_service_name(
                self._agent_id,
                input_name,
                target="local",
                strict=True,
            )
            logger.debug(
                f"[PERSPECTIVE] normalize input='{input_name}' -> local='{res.local_name}' method='{res.resolution_method}'"
            )
            return res.local_name
        except Exception as e:
            logger.error(f"[PERSPECTIVE] normalize_agent_local_name failed: {e}")
            raise

    async def delete_service_two_step(self, service_name: str) -> Dict[str, Any]:
        """
        两步删除服务：从配置文件删除 + 从Registry注销

        Args:
            service_name: 服务名称

        Returns:
            Dict: 包含两步操作结果的字典
        """
        result = {
            "step1_config_removal": False,
            "step2_registry_cleanup": False,
            "step1_error": None,
            "step2_error": None,
            "overall_success": False
        }

        # 第一步：从配置文件删除
        try:
            result["step1_config_removal"] = await self.delete_service_async(service_name)
            if not result["step1_config_removal"]:
                result["step1_error"] = "Failed to remove service from configuration"
        except Exception as e:
            result["step1_error"] = f"Configuration removal failed: {str(e)}"
            logger.error(f"Step 1 (config removal) failed: {e}")

        # 第二步：从Registry清理（即使第一步失败也尝试）
        try:
            if self._context_type == ContextType.STORE:
                # Store级别：清理global_agent_store的Registry
                cleanup_success = await self._store.orchestrator.registry.cleanup_service(service_name)
            else:
                # Agent级别：清理特定agent的Registry
                global_name = service_name
                if self._service_mapper:
                    global_name = self._service_mapper.to_global_name(service_name)
                cleanup_success = await self._store.orchestrator.registry.cleanup_service(global_name, self._agent_id)

            result["step2_registry_cleanup"] = cleanup_success
            if not cleanup_success:
                result["step2_error"] = "Failed to cleanup service from registry"
        except Exception as e:
            result["step2_error"] = f"Registry cleanup failed: {str(e)}"
            logger.warning(f"Step 2 (registry cleanup) failed: {e}")

        result["overall_success"] = result["step1_config_removal"] and result["step2_registry_cleanup"]
        return result

    def reset_config(self) -> bool:
        """
        重置配置（同步版本）
        
        清空所有 pykv 缓存数据和 mcp.json 文件。
        相当于批量执行 delete_service 操作。
        """
        return self._run_async_via_bridge(
            self.reset_config_async(),
            op_name="service_management.reset_config"
        )

    def switch_cache(self, cache_config: Any) -> bool:
        """运行时切换缓存后端（同步版本）。

        仅支持 Store 上下文；Agent 上下文会抛出 ValueError。
        """
        try:
            return self._run_async_via_bridge(
                self.switch_cache_async(cache_config),
                op_name="service_management.switch_cache"
            )
        except Exception as e:
            logger.error(f"[NEW_ARCH] switch_cache failed: {e}")
            return False

    async def switch_cache_async(self, cache_config: Any) -> bool:
        """运行时切换缓存后端（异步版本）。"""
        try:
            if self._context_type != ContextType.STORE:
                raise ValueError("Cache switching is only supported in STORE context")

            # 委托给 Store 层的封装方法，内部会进行配置解析和连接测试
            await self._store._switch_cache_backend(cache_config)
            return True
        except Exception as e:
            logger.error(f"Failed to switch cache backend: {e}")
            return False

    async def reset_config_async(self) -> bool:
        """
        重置配置（异步版本）
        
        清空所有 pykv 缓存数据和 mcp.json 文件。
        相当于批量执行 delete_service 操作。
        
        清理内容：
        - pykv 实体层：services, tools
        - pykv 关系层：agent_services, service_tools
        - pykv 状态层：service_status, service_metadata
        - mcp.json 文件
        - 健康检查任务（通过服务不存在检测自动停止）
        
        根据上下文类型执行不同的重置操作：
        - Store 上下文：清空所有 Agent 的配置
        - Agent 上下文：只清空该 Agent 的配置
        """
        if self._context_type == ContextType.STORE:
            return await self._reset_store_config()
        else:
            return await self._reset_agent_config()

    async def _reset_store_config(self) -> bool:
        """
        Store 级别重置配置
        
        清理流程：
        1. 获取所有 Agent ID
        2. 对每个 Agent 调用 registry.clear_async()
           - clear_async 内部调用 remove_service_async 逐个删除服务
           - remove_service_async 清理：实体层、关系层、状态层、工具实体
        3. 重置 mcp.json 为空配置
        """
        logger.info("[RESET_CONFIG] [STORE] Store level: starting to reset all configurations")
        
        # 1. 获取所有 Agent ID
        agent_ids = await self._store.registry.get_all_agent_ids_async()
        logger.debug(f"[RESET_CONFIG] [CLEAN] Found {len(agent_ids)} Agents need to be cleaned")
        
        # 2. 清空每个 Agent 的缓存数据
        for agent_id in agent_ids:
            logger.debug(f"[RESET_CONFIG] [CLEAN] Cleaning Agent: {agent_id}")
            await self._store.registry.clear_async(agent_id)
        
        # 3. 单源模式：不再重置 mcp.json，重置以 KV 为准
        logger.info("[RESET_CONFIG] [STORE] Single-source KV: skip mcp.json reset")
        
        logger.info("[RESET_CONFIG] [STORE] Store level: configuration reset completed")
        return True

    async def _reset_agent_config(self) -> bool:
        """Agent级别重置配置的内部实现"""
        try:
            logger.info(f"[RESET_CONFIG] [AGENT] Agent level: resetting all configurations for Agent {self._agent_id}")

            # 1. 清空Agent在缓存中的数据（使用异步版本）
            await self._store.registry.clear_async(self._agent_id)

            # 2. 单源模式：不再同步到分片文件
            logger.info("Single-source mode: skip shard mapping files sync")

            logger.info(f"[RESET_CONFIG] [AGENT] Agent level: Agent {self._agent_id} configuration reset completed")
            return True

        except Exception as e:
            logger.error(f"[RESET_CONFIG] [ERROR] Agent level configuration reset failed: {e}")
            return False

    def show_config(self) -> Dict[str, Any]:
        """
        显示配置信息（同步版本）

        - Store级别: 返回所有Agent的配置
        - Agent级别: 返回该Agent的配置

        Returns:
            Dict: 配置信息字典
        """
        try:
            return self._run_async_via_bridge(
                self.show_config_async(),
                op_name="service_management.show_config"
            )
        except Exception as e:
            logger.error(f"[NEW_ARCH] show_config failed: {e}")
            return {}

    async def show_config_async(self) -> Dict[str, Any]:
        """
        显示配置信息（异步版本）- 遵循 Functional Core, Imperative Shell 架构

        架构说明：
        - 使用 ShowConfigAsyncShell 作为异步外壳，负责 pykv IO 操作
        - 使用 ShowConfigLogicCore 作为纯逻辑核心，负责数据组装
        - 严格遵循 pykv 唯一真相数据源原则

        根据上下文类型执行不同的显示操作：
        - Store上下文：显示所有Agent的配置
        - Agent上下文：显示该Agent的配置

        Returns:
            Dict: 配置信息字典
        """
        try:
            # 获取 CacheLayerManager 实例
            cache_layer = self._get_cache_layer_manager()
            
            # 创建异步外壳实例
            from mcpstore.core.architecture.show_config_shell import ShowConfigAsyncShell
            shell = ShowConfigAsyncShell(cache_layer)
            
            if self._context_type == ContextType.STORE:
                return await shell.show_store_config_async()
            else:
                return await shell.show_agent_config_async(self._agent_id)
                
        except Exception as e:
            logger.error(f"Failed to show config: {e}")
            # 使用纯逻辑核心构建错误响应
            from mcpstore.core.architecture.show_config_core import ShowConfigLogicCore
            logic_core = ShowConfigLogicCore()
            return logic_core.build_error_response(
                f"Failed to show config: {str(e)}",
                agent_id=self._agent_id if self._context_type != ContextType.STORE else None
            )

    def _get_cache_layer_manager(self):
        """
        获取 CacheLayerManager 实例
        
        遵循 pykv 唯一真相数据源原则，确保使用正确的缓存层管理器。
        
        Returns:
            CacheLayerManager 实例
            
        Raises:
            RuntimeError: 如果无法获取 CacheLayerManager
        """
        # 从 registry 获取 _cache_layer_manager
        # 注意：不再使用 _cache_layer，因为它在 Redis 模式下是 RedisStore，没有所需的方法
        cache_layer = getattr(self._store.registry, '_cache_layer_manager', None)
        if cache_layer is not None:
            return cache_layer
        
        # 尝试从 store 获取
        cache_layer = getattr(self._store, '_cache_layer_manager', None)
        if cache_layer is not None:
            return cache_layer
        
        raise RuntimeError(
            "无法获取 CacheLayerManager 实例。"
            "请确保 MCPStore 已正确初始化，且 registry._cache_layer_manager 已设置。"
        )

    def delete_config(self, client_id_or_service_name: str) -> Dict[str, Any]:
        """
        删除服务配置（同步版本）

        Args:
            client_id_or_service_name: client_id或服务名

        Returns:
            Dict: 删除结果
        """
        try:
            return self._run_async_via_bridge(
                self.delete_config_async(client_id_or_service_name),
                op_name="service_management.delete_config"
            )
        except Exception as e:
            logger.error(f"[NEW_ARCH] delete_config failed: {e}")
            return {
                "success": False,
                "error": str(e),
                "client_id": None,
                "service_name": None
            }

    async def delete_config_async(self, client_id_or_service_name: str) -> Dict[str, Any]:
        """
        删除服务配置（异步版本）

        支持智能参数识别：
        - 如果传入client_id，直接使用
        - 如果传入服务名，自动查找对应的client_id
        - Agent级别严格隔离，只在指定agent范围内查找

        Args:
            client_id_or_service_name: client_id或服务名

        Returns:
            Dict: 删除结果
        """
        try:
            if self._context_type == ContextType.STORE:
                return await self._delete_store_config(client_id_or_service_name)
            else:
                return await self._delete_agent_config(client_id_or_service_name)
        except Exception as e:
            logger.error(f"Failed to delete config: {e}")
            return {
                "success": False,
                "error": f"Failed to delete config: {str(e)}",
                "client_id": None,
                "service_name": None
            }

    def update_config(self, client_id_or_service_name: str, new_config: Dict[str, Any]) -> Dict[str, Any]:
        """
        更新服务配置（同步版本）

        Args:
            client_id_or_service_name: client_id或服务名
            new_config: 新的配置信息

        Returns:
            Dict: 更新结果
        """
        try:
            return self._run_async_via_bridge(
                self.update_config_async(client_id_or_service_name, new_config),
                op_name="service_management.update_config"
            )
        except Exception as e:
            logger.error(f"[NEW_ARCH] update_config failed: {e}")
            return {
                "success": False,
                "error": str(e),
                "client_id": None,
                "service_name": None,
                "old_config": None,
                "new_config": None
            }

    async def update_config_async(self, client_id_or_service_name: str, new_config: Dict[str, Any]) -> Dict[str, Any]:
        """
        更新服务配置（异步版本）

        支持智能参数识别和多种配置格式：
        - 参数识别：client_id或服务名自动识别
        - 配置格式：支持简化格式和mcpServers格式
        - 字段验证：不允许修改服务名，不允许新增字段类型
        - Agent级别严格隔离

        Args:
            client_id_or_service_name: client_id或服务名
            new_config: 新的配置信息

        Returns:
            Dict: 更新结果
        """
        try:
            if self._context_type == ContextType.STORE:
                return await self._update_store_config(client_id_or_service_name, new_config)
            else:
                return await self._update_agent_config(client_id_or_service_name, new_config)
        except Exception as e:
            logger.error(f"Failed to update config: {e}")
            return {
                "success": False,
                "error": f"Failed to update config: {str(e)}",
                "client_id": None,
                "service_name": None,
                "old_config": None,
                "new_config": None
            }

    def _is_deterministic_client_id(self, identifier: str) -> bool:
        """使用 ClientIDGenerator 统一判断确定性client_id格式"""
        try:
            from mcpstore.core.utils.id_generator import ClientIDGenerator
            return ClientIDGenerator.is_deterministic_format(identifier)
        except Exception:
            return False

    def _parse_deterministic_client_id(self, client_id: str, agent_id: str) -> Tuple[str, str]:
        """使用 ClientIDGenerator 统一解析确定性client_id，并验证agent范围"""
        from mcpstore.core.utils.id_generator import ClientIDGenerator
        parsed = ClientIDGenerator.parse_client_id(client_id)
        if parsed.get("type") == "store":
            global_agent_store_id = self._store.client_manager.global_agent_store_id
            if agent_id != global_agent_store_id:
                raise ValueError(f"Store client_id '{client_id}' cannot be used with agent '{agent_id}'")
            return client_id, parsed.get("service_name")
        elif parsed.get("type") == "agent":
            if parsed.get("agent_id") != agent_id:
                raise ValueError(f"Client_id '{client_id}' belongs to agent '{parsed.get('agent_id')}', not '{agent_id}'")
            return client_id, parsed.get("service_name")
        raise ValueError(f"Cannot parse client_id format: {client_id}")

    async def _validate_resolved_mapping_async(self, client_id: str, service_name: str, agent_id: str) -> bool:
        """
        验证解析后的client_id和service_name映射是否有效（异步版本）

        Args:
            client_id: 解析出的client_id
            service_name: 解析出的service_name
            agent_id: Agent ID

        Returns:
            bool: 映射是否有效
        """
        try:
            # 检查client_id是否存在于agent的映射中 - 从 pykv 获取
            agent_clients = await self._store.registry.get_agent_clients_async(agent_id)
            if client_id not in agent_clients:
                logger.debug(f" [VALIDATE_MAPPING] client_id '{client_id}' not found in agent '{agent_id}' clients")
                return False

            # 检查service_name是否存在于Registry中
            existing_client_id = await self._store.registry._agent_client_service.get_service_client_id_async(agent_id, service_name)
            if existing_client_id != client_id:
                logger.debug(f" [VALIDATE_MAPPING] service '{service_name}' maps to different client_id: expected={client_id}, actual={existing_client_id}")
                return False

            return True
        except Exception as e:
            logger.debug(f" [VALIDATE_MAPPING] Validation failed: {e}")
            return False

    def _validate_resolved_mapping(self, client_id: str, service_name: str, agent_id: str) -> bool:
        raise RuntimeError("[SERVICE_MANAGEMENT] Synchronous validate_mapping is disabled, please use _validate_resolved_mapping_async.")

    async def _resolve_client_id_async(self, client_id_or_service_name: str, agent_id: str) -> Tuple[str, str]:
        """
        智能解析client_id或服务名（使用最新的确定性算法）

        Args:
            client_id_or_service_name: 用户输入的参数
            agent_id: Agent ID（用于范围限制）

        Returns:
            Tuple[client_id, service_name]: 解析后的client_id和服务名

        Raises:
            ValueError: 当参数无法解析或不存在时
        """
        logger.debug(f"[RESOLVE_CLIENT_ID] start value='{client_id_or_service_name}' agent='{agent_id}'")

        from .agent_service_mapper import AgentServiceMapper
        global_agent_id = self._store.client_manager.global_agent_store_id

        # 1) 优先：确定性 client_id 直接解析
        if self._is_deterministic_client_id(client_id_or_service_name):
            try:
                client_id, service_name = self._parse_deterministic_client_id(client_id_or_service_name, agent_id)
                logger.debug(f"[RESOLVE_CLIENT_ID] deterministic_ok client_id={client_id} service_name={service_name}")
                return client_id, service_name
            except ValueError as e:
                logger.debug(f"[RESOLVE_CLIENT_ID] deterministic_parse_failed error={e}")
                # 继续按服务名处理

        # 2) Agent 模式：透明代理到 Store（不依赖 Agent 命名空间缓存）
        if self._context_type == ContextType.AGENT and agent_id != global_agent_id:
            # 2.1 判断输入是本地名还是全局名
            input_name = client_id_or_service_name
            global_service_name = None

            if AgentServiceMapper.is_any_agent_service(input_name):
                # 输入是全局名，校验归属
                try:
                    parsed_agent_id, local_name = AgentServiceMapper.parse_agent_service_name(input_name)
                    if parsed_agent_id != agent_id:
                        raise ValueError(f"Service '{input_name}' belongs to agent '{parsed_agent_id}', not '{agent_id}'")
                    global_service_name = input_name
                except ValueError as e:
                    raise ValueError(f"Invalid agent service name '{input_name}': {e}")
            else:
                # 输入是本地名：优先用映射，其次用规则推导
                mapped = await self._store.registry.get_global_name_from_agent_service_async(agent_id, input_name)
                global_service_name = mapped or AgentServiceMapper(agent_id).to_global_name(input_name)

            # 2.2 优先在 Agent 命名空间解析 client_id，再回退到 Store 命名空间
            client_id = await self._store.registry._agent_client_service.get_service_client_id_async(agent_id, input_name)
            if not client_id:
                # 回退到 Store 命名空间
                client_id = await self._store.registry._agent_client_service.get_service_client_id_async(global_agent_id, global_service_name)

            if not client_id:
                available_agent = ', '.join(await self._store.registry.get_all_service_names_async(agent_id)) or 'None'
                available_global = ', '.join(await self._store.registry.get_all_service_names_async(global_agent_id)) or 'None'
                raise ValueError(
                    f"Service '{input_name}' (global '{global_service_name}') not found. "
                    f"Agent services: {available_agent}. Store services: {available_global}"
                )

            logger.debug(f"[RESOLVE_CLIENT_ID] agent_proxy_ok local_or_global='{input_name}' -> global='{global_service_name}' client_id={client_id}")
            return client_id, global_service_name

        # 3) Store 模式：直接在 Store 命名空间解析
        service_name = client_id_or_service_name
        service_names = await self._store.registry.get_all_service_names_async(agent_id)
        if service_name in service_names:
            client_id = await self._store.registry._agent_client_service.get_service_client_id_async(agent_id, service_name)
            if client_id:
                logger.debug(f"[RESOLVE_CLIENT_ID] store_lookup_ok service={service_name} client_id={client_id}")
                return client_id, service_name
            else:
                raise ValueError(f"Service '{service_name}' found but no client_id mapping")

        available_services = ', '.join(service_names) if service_names else 'None'
        raise ValueError(f"Service '{service_name}' not found in store. Available services: {available_services}")

    def _resolve_client_id(self, client_id_or_service_name: str, agent_id: str) -> Tuple[str, str]:
        """
        同步包装，保留给旧代码使用；内部通过 AOB 执行异步解析。
        """
        return self._run_async_via_bridge(
            self._resolve_client_id_async(client_id_or_service_name, agent_id),
            op_name="service_management.resolve_client_id"
        )

    async def _delete_store_config(self, client_id_or_service_name: str) -> Dict[str, Any]:
        """Store级别删除配置的内部实现"""
        try:
            logger.info(f"[DELETE_CONFIG] [STORE] Store level: deleting configuration {client_id_or_service_name}")

            global_agent_store_id = self._store.client_manager.global_agent_store_id

            # 解析client_id和服务名
            client_id, service_name = await self._resolve_client_id_async(client_id_or_service_name, global_agent_store_id)

            logger.info(f"[DELETE_CONFIG] [RESOLVE] Resolution result: client_id={client_id}, service_name={service_name}")

            # 验证服务存在
            if not self._store.registry.get_session(global_agent_store_id, service_name):
                logger.warning(f"Service {service_name} not found in registry, but continuing with cleanup")

            # 事务性删除：直接删除缓存/映射，不触碰文件
            # 1. 从缓存中删除服务（包括工具和会话）- 使用异步版本
            await self._store.registry.remove_service_async(global_agent_store_id, service_name)

            # 2. 删除Service-Client映射
            self._store.registry.remove_service_client_mapping(global_agent_store_id, service_name)

            # 3. 删除Client配置
            self._store.registry.remove_client_config(client_id)

            # 4. 删除Agent-Client映射
            self._store.registry.remove_agent_client_mapping(global_agent_store_id, client_id)

            # 5. 单源模式：不再同步到分片文件或 mcp.json
            logger.info("Single-source mode: skip file/shard sync")

            logger.info(f"[DELETE_CONFIG] [STORE] Store level: configuration deletion completed {service_name}")

            return {
                "success": True,
                "message": f"Service '{service_name}' deleted successfully",
                "client_id": client_id,
                "service_name": service_name
            }

        except Exception as e:
            logger.error(f"[DELETE_CONFIG] [ERROR] Store level configuration deletion failed: {e}")
            return {
                "success": False,
                "error": f"Failed to delete store config: {str(e)}",
                "client_id": None,
                "service_name": None
            }

    async def _delete_agent_config(self, client_id_or_service_name: str) -> Dict[str, Any]:
        """Agent级别删除配置的内部实现"""
        try:
            logger.info(f"[DELETE_CONFIG] [AGENT] Agent level: deleting Agent {self._agent_id} configuration {client_id_or_service_name}")

            # 解析client_id和服务名
            client_id, service_name = await self._resolve_client_id_async(client_id_or_service_name, self._agent_id)

            logger.info(f"[DELETE_CONFIG] [RESOLVE] Resolution result: client_id={client_id}, service_name={service_name}")

            # 验证服务存在
            if not self._store.registry.get_session(self._agent_id, service_name):
                logger.warning(f"Service {service_name} not found in registry for agent {self._agent_id}, but continuing with cleanup")

            # Agent级别删除：只删除缓存，不修改mcp.json
            # 1. 从缓存中删除服务（包括工具和会话）- 使用异步版本
            await self._store.registry.remove_service_async(self._agent_id, service_name)

            # 2. 删除Service-Client映射
            self._store.registry.remove_service_client_mapping(self._agent_id, service_name)

            # 3. 删除Client配置
            self._store.registry.remove_client_config(client_id)

            # 4. 删除Agent-Client映射
            self._store.registry.remove_agent_client_mapping(self._agent_id, client_id)

            # 5. 单源模式：不再同步到分片文件
            logger.info("Single-source mode: skip shard mapping files sync")

            logger.info(f"[DELETE_CONFIG] [AGENT] Agent level: configuration deletion completed {service_name}")

            return {
                "success": True,
                "message": f"Service '{service_name}' deleted successfully from agent '{self._agent_id}'",
                "client_id": client_id,
                "service_name": service_name
            }

        except Exception as e:
            logger.error(f"[DELETE_CONFIG] [ERROR] Agent level configuration deletion failed: {e}")
            return {
                "success": False,
                "error": f"Failed to delete agent config: {str(e)}",
                "client_id": None,
                "service_name": None
            }

    def _validate_and_normalize_config(self, new_config: Dict[str, Any], service_name: str, old_config: Dict[str, Any]) -> Dict[str, Any]:
        """
        验证和标准化配置

        Args:
            new_config: 新配置
            service_name: 服务名
            old_config: 原配置

        Returns:
            Dict: 标准化后的配置

        Raises:
            ValueError: 配置验证失败
        """
        # 1. 处理配置格式
        if "mcpServers" in new_config:
            # mcpServers格式
            if len(new_config["mcpServers"]) != 1:
                raise ValueError("mcpServers format must contain exactly one service")

            config_service_name = list(new_config["mcpServers"].keys())[0]
            if config_service_name != service_name:
                raise ValueError(f"Cannot change service name from '{service_name}' to '{config_service_name}'")

            normalized_config = new_config["mcpServers"][service_name]
        else:
            # 简化格式
            if "name" in new_config:
                raise ValueError("Cannot modify service name in config update")
            normalized_config = new_config.copy()

        # 2. 验证字段类型一致性
        old_config_keys = set(old_config.keys())
        new_config_keys = set(normalized_config.keys())

        # 检查是否有新增的字段类型
        new_fields = new_config_keys - old_config_keys
        if new_fields:
            raise ValueError(f"Cannot add new field types: {list(new_fields)}. Only existing fields can be updated.")

        # 3. 验证字段值的合理性
        for key, value in normalized_config.items():
            if key in old_config:
                old_type = type(old_config[key])
                new_type = type(value)

                # 允许的类型转换
                if old_type != new_type:
                    # 允许字符串和数字之间的转换
                    if not ((old_type in [str, int, float] and new_type in [str, int, float]) or
                            (old_type == list and new_type == list)):
                        raise ValueError(f"Field '{key}' type mismatch: expected {old_type.__name__}, got {new_type.__name__}")

        return normalized_config

    async def _update_store_config(self, client_id_or_service_name: str, new_config: Dict[str, Any]) -> Dict[str, Any]:
        """Store级别更新配置的内部实现"""
        try:
            logger.info(f"[UPDATE_CONFIG] [STORE] Store level: updating configuration {client_id_or_service_name}")

            global_agent_store_id = self._store.client_manager.global_agent_store_id

            # 解析client_id和服务名
            client_id, service_name = await self._resolve_client_id_async(client_id_or_service_name, global_agent_store_id)

            logger.info(f"[UPDATE_CONFIG] [RESOLVE] Resolution result: client_id={client_id}, service_name={service_name}")

            # 获取当前配置
            old_complete_info = await self._store.registry.get_complete_service_info_async(global_agent_store_id, service_name)
            old_config = old_complete_info.get("config", {})

            if not old_config:
                raise ValueError(f"Service '{service_name}' configuration not found")

            # 验证和标准化新配置
            normalized_config = self._validate_and_normalize_config(new_config, service_name, old_config)

            logger.info(f"[UPDATE_CONFIG] [VALIDATE] Configuration validation passed, starting update: {service_name}")

            # 1. 清空服务的工具和会话数据
            self._store.registry.clear_service_tools_only(global_agent_store_id, service_name)

            # 2. 更新Client配置缓存
            self._store.registry.update_client_config(client_id, {
                "mcpServers": {service_name: normalized_config}
            })

            # 3. 设置服务状态为STARTUP并更新元数据
            from mcpstore.core.models.service import ServiceConnectionState
            await self._store.orchestrator.lifecycle_manager._transition_state(
                agent_id=global_agent_store_id,
                service_name=service_name,
                new_state=ServiceConnectionState.STARTUP,
                reason="config_updated",
                source="ServiceManagement",
            )

            # 从 pykv 异步获取并更新服务元数据中的配置
            metadata = await self._store.registry._service_state_service.get_service_metadata_async(global_agent_store_id, service_name)
            if metadata:
                metadata.service_config = normalized_config
                metadata.consecutive_failures = 0
                metadata.error_message = None
                from datetime import datetime
                metadata.state_entered_time = datetime.now()
                self._store.registry.set_service_metadata(global_agent_store_id, service_name, metadata)

            # 4. 单源模式：不再同步到 mcp.json 或分片文件
            logger.info("Single-source mode: skip file/shard sync")

            # 5. 触发生命周期管理器重新初始化服务
            await self._store.orchestrator.lifecycle_manager.initialize_service(
                global_agent_store_id, service_name, normalized_config
            )

            logger.info(f"[UPDATE_CONFIG] [STORE] Store level: configuration update completed {service_name}")

            return {
                "success": True,
                "message": f"Service '{service_name}' configuration updated successfully",
                "client_id": client_id,
                "service_name": service_name,
                "old_config": old_config,
                "new_config": normalized_config
            }

        except Exception as e:
            logger.error(f"[UPDATE_CONFIG] [ERROR] Store level configuration update failed: {e}")
            return {
                "success": False,
                "error": f"Failed to update store config: {str(e)}",
                "client_id": None,
                "service_name": None,
                "old_config": None,
                "new_config": None
            }

    async def _update_agent_config(self, client_id_or_service_name: str, new_config: Dict[str, Any]) -> Dict[str, Any]:
        """Agent级别更新配置的内部实现"""
        try:
            logger.info(f"[UPDATE_CONFIG] [AGENT] Agent level: updating Agent {self._agent_id} configuration {client_id_or_service_name}")

            # 解析client_id和服务名
            client_id, service_name = await self._resolve_client_id_async(client_id_or_service_name, self._agent_id)

            logger.info(f"[UPDATE_CONFIG] [RESOLVE] Resolution result: client_id={client_id}, service_name={service_name}")

            # 获取当前配置
            old_complete_info = await self._store.registry.get_complete_service_info_async(self._agent_id, service_name)
            old_config = old_complete_info.get("config", {})

            if not old_config:
                raise ValueError(f"Service '{service_name}' configuration not found")

            # 验证和标准化新配置
            normalized_config = self._validate_and_normalize_config(new_config, service_name, old_config)

            logger.info(f"[UPDATE_CONFIG] [VALIDATE] Configuration validation passed, starting update: {service_name}")

            # 1. 清空服务的工具和会话数据
            self._store.registry.clear_service_tools_only(self._agent_id, service_name)

            # 2. 更新Client配置缓存
            self._store.registry.update_client_config(client_id, {
                "mcpServers": {service_name: normalized_config}
            })

            # 3. 设置服务状态为STARTUP并更新元数据
            from mcpstore.core.models.service import ServiceConnectionState
            await self._store.orchestrator.lifecycle_manager._transition_state(
                agent_id=self._agent_id,
                service_name=service_name,
                new_state=ServiceConnectionState.STARTUP,
                reason="agent_config_updated",
                source="ServiceManagement",
            )

            # 从 pykv 异步获取并更新服务元数据中的配置
            metadata = await self._store.registry._service_state_service.get_service_metadata_async(self._agent_id, service_name)
            if metadata:
                metadata.service_config = normalized_config
                metadata.consecutive_failures = 0
                metadata.error_message = None
                from datetime import datetime
                metadata.state_entered_time = datetime.now()
                self._store.registry.set_service_metadata(self._agent_id, service_name, metadata)

            # 4. 单源模式：不再同步到分片文件（Agent级别不更新mcp.json）
            logger.info("Single-source mode: skip shard mapping files sync")

            # 5. 触发生命周期管理器重新初始化服务
            await self._store.orchestrator.lifecycle_manager.initialize_service(
                self._agent_id, service_name, normalized_config
            )

            logger.info(f"[UPDATE_CONFIG] [AGENT] Agent level: configuration update completed {service_name}")

            return {
                "success": True,
                "message": f"Service '{service_name}' configuration updated successfully for agent '{self._agent_id}'",
                "client_id": client_id,
                "service_name": service_name,
                "old_config": old_config,
                "new_config": normalized_config
            }

        except Exception as e:
            logger.error(f"[UPDATE_CONFIG] [ERROR] Agent level configuration update failed: {e}")
            return {
                "success": False,
                "error": f"Failed to update agent config: {str(e)}",
                "client_id": None,
                "service_name": None,
                "old_config": None,
                "new_config": None
            }

    def get_service_status(self, name: str) -> dict:
        """获取单个服务的状态信息（同步版本，内部桥接异步）。"""
        try:
            return self._run_async_via_bridge(
                self.get_service_status_async(name),
                op_name="service_management.get_service_status"
            )
        except Exception as e:
            logger.error(f"[NEW_ARCH] get_service_status failed: {e}")
            return {"status": "error", "error": str(e)}

    async def get_service_status_async(self, name: str) -> dict:
        """获取单个服务的状态信息"""
        try:
            if self._context_type == ContextType.STORE:
                from mcpstore.utils.perspective_resolver import PerspectiveResolver

                resolver = PerspectiveResolver()
                name_res = resolver.normalize_service_name(
                    self._store.client_manager.global_agent_store_id,
                    name,
                    target="global",
                    strict=True,
                )
                return await self._store.orchestrator.get_service_status_async(name_res.global_name)
            else:
                # Agent模式：转换服务名称（严格校验）
                from mcpstore.utils.perspective_resolver import PerspectiveResolver

                resolver = PerspectiveResolver()
                name_res = resolver.normalize_service_name(
                    self._agent_id,
                    name,
                    target="global",
                    strict=True,
                )
                return await self._store.orchestrator.get_service_status_async(name_res.global_name)
        except Exception as e:
            logger.error(f"Failed to get service status for {name}: {e}")
            return {"status": "error", "error": str(e)}

    # 别名：符合命名规范
    def service_status(self, name: str) -> dict:
        return self.get_service_status(name)

    async def service_status_async(self, name: str) -> dict:
        return await self.get_service_status_async(name)

    def restart_service(self, name: str) -> bool:
        raise RuntimeError("[SERVICE_MANAGEMENT] Synchronous restart_service is disabled, please use restart_service_async.")

    async def restart_service_async(self, name: str) -> bool:
        """重启指定服务（透明代理）"""
        try:
            if self._context_type == ContextType.STORE:
                from mcpstore.utils.perspective_resolver import PerspectiveResolver
                from mcpstore.core.exceptions import PerspectiveResolutionError

                try:
                    resolver = PerspectiveResolver()
                    name_res = resolver.normalize_service_name(
                        self._store.client_manager.global_agent_store_id,
                        name,
                        target="global",
                        strict=False,
                    )
                    target_name = name_res.global_name
                except Exception as e:
                    raise PerspectiveResolutionError(name, str(e)) from e

                return await self._store.orchestrator.restart_service(target_name)
            else:
                # Agent模式：透明代理 - 将本地服务名映射到全局服务名，并在全局命名空间执行重启
                global_name = await self._map_agent_service_to_global(name)
                global_agent = self._store.client_manager.global_agent_store_id
                return await self._store.orchestrator.restart_service(global_name, global_agent)
        except Exception as e:
            logger.error(f"Failed to restart service {name}: {e}")
            raise

    # === Lifecycle-only disconnection (no config/registry deletion) ===
    def disconnect_service(self, name: str, reason: str = "user_requested") -> bool:
        raise RuntimeError("[SERVICE_MANAGEMENT] Synchronous disconnect_service is disabled, please use disconnect_service_async.")

    async def disconnect_service_async(self, name: str, reason: str = "user_requested") -> bool:
        """
        断开服务（异步版本）- 仅生命周期断链：不改配置/不删注册表。

        Store 上下文：name 视为全局名；
        Agent 上下文：自动将本地名映射为全局名后断开。
        """
        try:
            global_agent_id = self._store.client_manager.global_agent_store_id
            if self._context_type == ContextType.STORE:
                from mcpstore.utils.perspective_resolver import PerspectiveResolver
                from mcpstore.core.exceptions import PerspectiveResolutionError

                try:
                    resolver = PerspectiveResolver()
                    name_res = resolver.normalize_service_name(
                        self._store.client_manager.global_agent_store_id,
                        name,
                        target="global",
                        strict=False,
                    )
                    global_name = name_res.global_name
                except Exception as e:
                    raise PerspectiveResolutionError(name, str(e)) from e
            else:
                global_name = await self._map_agent_service_to_global(name)

            # 调用生命周期管理器执行优雅断开
            lm = self._store.orchestrator.lifecycle_manager
            await lm.graceful_disconnect(global_agent_id, global_name, reason)

            # 清空工具展示缓存（仅清工具，不删除服务实体）
            try:
                self._store.registry.clear_service_tools_only(global_agent_id, global_name)
            except Exception:
                pass
            return True
        except Exception as e:
            logger.error(f"[DISCONNECT_SERVICE] Failed to disconnect '{name}': {e}")
            raise

    # ===  新增：Agent 透明代理辅助方法 ===

    async def _map_agent_service_to_global(self, local_name: str) -> str:
        """
        将 Agent 的本地服务名映射到全局服务名

        Args:
            local_name: Agent 中的本地服务名

        Returns:
            str: 全局服务名
        """
        try:
            from mcpstore.utils.perspective_resolver import PerspectiveResolver

            resolver = PerspectiveResolver()
            res = resolver.normalize_service_name(
                self._agent_id,
                local_name,
                target="global",
                strict=False,
            )
            logger.debug(f" [SERVICE_PROXY] Service name mapping via PerspectiveResolver: {local_name} -> {res.global_name}")
            return res.global_name
        except Exception as e:
            logger.error(f" [SERVICE_PROXY] Service name mapping failed: {e}")
            return local_name

    async def _delete_store_service_with_sync(self, service_name: str):
        """Store 服务删除（带双向同步）"""
        try:
            # 1. 从 Registry 中删除（使用异步版本）
            await self._store.registry.remove_service_async(
                self._store.client_manager.global_agent_store_id,
                service_name
            )

            # 2. 单源模式：不再同步到 mcp.json
            logger.info("[SERVICE_DELETE] [STORE] Skip file sync (single-source KV)")

            # 3. 触发双向同步（如果是 Agent 服务）
            if hasattr(self._store, 'bidirectional_sync_manager'):
                await self._store.bidirectional_sync_manager.handle_service_deletion_with_sync(
                    self._store.client_manager.global_agent_store_id,
                    service_name
                )

        except Exception as e:
            logger.error(f" [SERVICE_DELETE] Store service deletion failed {service_name}: {e}")
            raise

    async def _delete_agent_service_with_sync(self, local_name: str):
        """Agent 服务删除（带双向同步），返回是否成功"""
        try:
            # 使用 PerspectiveResolver 统一归一化输入到本地名
            from mcpstore.utils.perspective_resolver import PerspectiveResolver

            resolver = PerspectiveResolver()
            local_res = resolver.normalize_service_name(
                self._agent_id,
                local_name,
                target="local",
                strict=True,
            )
            local_name = local_res.local_name

            success = True
            # 1. 获取全局名称（使用异步版本，避免 AOB 事件循环冲突）
            global_res = resolver.normalize_service_name(
                self._agent_id,
                local_name,
                target="global",
                strict=True,
            )
            global_name = global_res.global_name
            if not global_name:
                raise ValueError(f"Service mapping not found: {self._agent_id}:{local_name}")

            # 2. 从 Agent 缓存中删除（使用异步版本）
            await self._store.registry.remove_service_async(self._agent_id, local_name)

            # 3. 从 Store 缓存中删除（使用异步版本）
            await self._store.registry.remove_service_async(
                self._store.client_manager.global_agent_store_id,
                global_name
            )

            # 4. 移除映射关系（仅映射表，不触发关系/状态删除）
            await self._store.registry.remove_agent_service_mapping_async(self._agent_id, local_name)

            # 5. 单源模式：不再同步到 mcp.json
            logger.info("[SERVICE_DELETE] [AGENT] Skip file sync (single-source KV)")

            # 6. 清理服务状态数据
            try:
                state_manager = self._store.registry._cache_state_manager
                await state_manager.delete_service_status(global_name)
                await state_manager.delete_service_metadata(global_name)
                logger.info(
                    f"[SERVICE_DELETE] Service status cleanup successful: "
                    f"agent_id={self._agent_id}, service={local_name}, global_name={global_name}"
                )
            except Exception as cleanup_error:
                logger.error(
                    f"[SERVICE_DELETE] Service status cleanup failed: "
                    f"agent_id={self._agent_id}, service={local_name}, error={cleanup_error}"
                )
                raise

            # 7. 单源模式：不再同步到分片文件
            logger.info("Single-source mode: skip shard mapping files sync")
            return success

        except Exception as e:
            logger.error(f" [SERVICE_DELETE] Agent service deletion failed {self._agent_id}:{local_name}: {e}")
            raise

    def show_mcpconfig(self) -> Dict[str, Any]:
        """
        根据当前上下文（store/agent）获取对应的配置信息

        Returns:
            Dict[str, Any]: Store上下文返回MCP JSON格式，Agent上下文返回client配置字典
        """
        if self._context_type == ContextType.STORE:
            # Store上下文：返回MCP JSON格式的配置（从缓存读取，更高效）
            try:
                config = self._store._unified_config.get_mcp_config()
                # 确保返回格式正确
                if isinstance(config, dict) and 'mcpServers' in config:
                    return config
                else:
                    logger.warning("Invalid MCP config format")
                    return {"mcpServers": {}}
            except Exception as e:
                logger.error(f"Failed to show MCP config: {e}")
                return {"mcpServers": {}}
        else:
            # Agent上下文：返回所有相关client配置的字典
            return self._run_async_via_bridge(
                self._show_config_agent_async(),
                op_name="service_management.show_config_agent"
            )

    async def _show_config_agent_async(self) -> Dict[str, Any]:
        """Agent上下文的 show_config 异步实现"""
        agent_id = self._agent_id
        # 从 pykv 获取 client_ids
        client_ids = await self._store.registry.get_agent_clients_async(agent_id)

        # 获取每个client的配置
        result = {}
        for client_id in client_ids:
            client_config = self._store.orchestrator.client_manager.get_client_config(client_id)
            if client_config:
                result[client_id] = client_config

        return result

    def wait_service(self, client_id_or_service_name: str,
                    status: Union[str, List[str]] = 'healthy',
                    timeout: float = 10.0,
                    raise_on_timeout: bool = False) -> Dict[str, Any]:
        """
        等待服务达到指定状态（同步版本，使用新架构避免死锁）。

        Args:
            client_id_or_service_name: client_id或服务名（智能识别）
            status: 目标状态，可以是单个状态字符串或状态列表
            timeout: 超时时间（秒），默认10秒
            raise_on_timeout: 超时时是否抛出异常，默认False

        Returns:
            dict: {success, status, window_metrics, retry_in, hard_timeout_in, lease_remaining,
                   next_retry_time, hard_deadline, lease_deadline, response_time,
                   consecutive_failures, last_state_change, hard_timeout_remaining, last_error}

        Raises:
            TimeoutError: 当raise_on_timeout=True且超时时抛出
            ValueError: 当参数无法解析时抛出
        """
        try:
            # 解析服务名称（简化版，实际可能需要更复杂的解析逻辑）
            service_name = self._extract_service_name_from_identifier(client_id_or_service_name)

            # 使用新架构：同步外壳
            if not hasattr(self, '_service_management_sync_shell'):
                from ..architecture import ServiceManagementFactory
                self._service_management_sync_shell, _, _ = ServiceManagementFactory.create_service_management(
                    self._store.registry,
                    self._store.orchestrator,
                    agent_id=self._agent_id or self._store.client_manager.global_agent_store_id
                )

            # 直接调用同步外壳，避免_sync_helper.run_async的复杂性
            result = self._service_management_sync_shell.wait_service(service_name, timeout)
            if isinstance(result, bool):
                result = {"success": bool(result), "status": None}
            result = self._enrich_wait_result(result)
            if not result.get("success") and raise_on_timeout:
                raise TimeoutError(f"Service {service_name} did not reach status {status} within {timeout} seconds")
            return result

        except Exception as e:
            logger.error(f"[NEW_ARCH] wait_service failed: {e}")
            if raise_on_timeout:
                raise
            return {"success": False, "error": str(e)}

    async def wait_service_async(self, client_id_or_service_name: str,
                               status: Union[str, List[str]] = 'healthy',
                               timeout: float = 10.0,
                               raise_on_timeout: bool = False) -> Dict[str, Any]:
        """
        等待服务达到指定状态（异步版本）

        Args:
            client_id_or_service_name: client_id或服务名（智能识别）
            status: 目标状态，可以是单个状态字符串或状态列表
            timeout: 超时时间（秒），默认10秒
            raise_on_timeout: 超时时是否抛出异常，默认False

        Returns:
            dict: {success, status, window_metrics, retry_in, hard_timeout_in, lease_remaining,
                   next_retry_time, hard_deadline, lease_deadline, response_time,
                   consecutive_failures, last_state_change, hard_timeout_remaining, last_error}

        Raises:
            TimeoutError: 当raise_on_timeout=True且超时时抛出
            ValueError: 当参数无法解析时抛出
        """
        try:
            # 解析参数
            agent_scope = self._agent_id if self._context_type == ContextType.AGENT else self._store.client_manager.global_agent_store_id
            client_id, service_name = await self._resolve_client_id_async(client_id_or_service_name, agent_scope)

            # 在纯视图模式下，Agent 的状态查询统一使用全局命名空间
            status_agent_key = self._store.client_manager.global_agent_store_id


            # 诊断：解析后的作用域与标识
            try:
                logger.info(f"[WAIT_SERVICE] resolved agent_scope={agent_scope} client_id='{client_id}' service='{service_name}' status_agent_key={status_agent_key}")
            except Exception:
                pass

            # 解析等待模式
            change_mode = False
            if isinstance(status, str) and status.lower() == 'change':
                change_mode = True
                logger.info(f"[WAIT_SERVICE] start mode=change service='{service_name}' timeout={timeout}s")
                try:
                    initial_status = (await self._store.orchestrator.get_service_status_async(service_name, status_agent_key) or {}).get("status", "unknown")
                except Exception as _e_init:
                    logger.debug(f"[WAIT_SERVICE] initial_status_error service='{service_name}' error={_e_init}")
                    initial_status = "unknown"
            else:
                # 规范化目标状态
                target_statuses = self._normalize_target_statuses(status)
                logger.info(f"[WAIT_SERVICE] start mode=target service='{service_name}' client_id='{client_id}' target={target_statuses} timeout={timeout}s")

            start_time = time.time()
            poll_interval = 0.2  # 200ms轮询间隔
            prev_status = None
            last_log = start_time
            last_meta: Dict[str, Any] = {}

            while True:
                # 检查超时
                elapsed = time.time() - start_time
                if elapsed >= timeout:
                    if change_mode:
                        msg = f"[WAIT_SERVICE] timeout mode=change service='{service_name}' from='{initial_status}' elapsed={elapsed:.2f}s"
                    else:
                        msg = f"[WAIT_SERVICE] timeout mode=target service='{service_name}' target={target_statuses} last='{prev_status}' elapsed={elapsed:.2f}s"
                    logger.warning(msg)
                    if raise_on_timeout:
                        raise TimeoutError(msg)
                    return self._enrich_wait_result({
                        "success": False,
                        "status": prev_status or "unknown",
                        "hard_timeout_remaining": max(timeout - elapsed, 0.0),
                        "last_error": msg,
                    }, meta=last_meta)

                # 获取当前状态（先读一次缓存，随后在必要时读一次新缓存以防止竞态）
                try:

                    status_dict = await self._store.orchestrator.get_service_status_async(service_name, status_agent_key) or {}
                    current_status = status_dict.get("status", "unknown")
                    try:
                        meta = await self._store.registry._service_state_service.get_service_metadata_async(status_agent_key, service_name)
                        if meta:
                            last_meta = {
                                "window_error_rate": getattr(meta, "window_error_rate", None),
                                "latency_p95": getattr(meta, "latency_p95", None),
                                "latency_p99": getattr(meta, "latency_p99", None),
                                "sample_size": getattr(meta, "sample_size", None),
                                "next_retry_time": getattr(meta, "next_retry_time", None),
                                "hard_deadline": getattr(meta, "hard_deadline", None),
                                "lease_deadline": getattr(meta, "lease_deadline", None),
                                "response_time": getattr(meta, "response_time", None),
                                "consecutive_failures": getattr(meta, "consecutive_failures", None),
                                "last_state_change": getattr(meta, "state_entered_time", None) or getattr(meta, "last_health_check", None),
                                "last_error": getattr(meta, "error_message", None),
                            }
                    except Exception as meta_err:
                        logger.debug(f"[WAIT_SERVICE] metadata_fetch_error service='{service_name}' error={meta_err}")

                    # 仅在状态变化或每2秒节流一次打印
                    now = time.time()
                    if current_status != prev_status or (now - last_log) > 2.0:
                        logger.debug(f"[WAIT_SERVICE] status service='{service_name}' value='{current_status}'")
                        # 对比 orchestrator 与 registry 的状态及最近健康检查（节流打印）
                        try:
                            reg_state = await self._store.registry.get_service_state_async(status_agent_key, service_name)
                            meta = await self._store.registry._service_state_service.get_service_metadata_async(status_agent_key, service_name)
                            last_check_ts = meta.last_health_check.isoformat() if getattr(meta, 'last_health_check', None) else None
                            logger.debug(f"[WAIT_SERVICE] compare orchestrator='{current_status}' registry='{getattr(reg_state,'value',reg_state)}' last_check={last_check_ts}")
                        except Exception as e:
                            logger.debug(f"[WAIT_SERVICE] Failed to get metadata: {e}")

                        prev_status, last_log = current_status, now

                    if change_mode:
                        if current_status != initial_status:
                            logger.info(f"[WAIT_SERVICE] done mode=change service='{service_name}' from='{initial_status}' to='{current_status}' elapsed={elapsed:.2f}s")
                            return self._enrich_wait_result({
                                "success": True,
                                "status": current_status,
                                "last_error": last_meta.get("last_error") if last_meta else None,
                            }, meta=last_meta)
                    else:
                        # 检查是否达到目标状态
                        if current_status in target_statuses:
                            logger.info(f"[WAIT_SERVICE] done mode=target service='{service_name}' reached='{current_status}' elapsed={elapsed:.2f}s")
                            return self._enrich_wait_result({
                                "success": True,
                                "status": current_status,
                                "last_error": last_meta.get("last_error") if last_meta else None,
                            }, meta=last_meta)
                except Exception as e:
                    # 降级到 debug，避免无意义刷屏
                    logger.debug(f"[WAIT_SERVICE] status_error service='{service_name}' error={e}")
                    # 继续轮询

                # 等待下次轮询
                await asyncio.sleep(poll_interval)

        except ValueError as e:
            logger.error(f"[WAIT_SERVICE] param_error error={e}")
            raise
        except Exception as e:
            logger.error(f"[WAIT_SERVICE] unexpected_error error={e}")
            if raise_on_timeout:
                raise
            return {"success": False, "error": str(e)}

    @staticmethod
    def _remaining_seconds(dt: Any) -> Optional[float]:
        """计算距离未来时间的剩余秒数"""
        try:
            if dt is None:
                return None
            import datetime
            if isinstance(dt, (int, float)):
                return max(dt - time.time(), 0.0)
            if isinstance(dt, datetime.datetime):
                return max((dt - datetime.datetime.now()).total_seconds(), 0.0)
            return None
        except Exception:
            return None

    @staticmethod
    def _serialize_time(dt: Any) -> Any:
        """统一序列化时间字段为 ISO 字符串或原值"""
        try:
            if isinstance(dt, datetime.datetime):
                return dt.isoformat()
            return dt
        except Exception:
            return dt

    def _enrich_wait_result(self, result: Dict[str, Any], meta: Any = None) -> Dict[str, Any]:
        """
        将 wait_service 返回结构补齐到与健康状态对齐：
        - window_metrics（错误率/延迟分位/样本数）
        - retry_in/hard_timeout_in/lease_remaining
        - next_retry_time/hard_deadline/lease_deadline（ISO 字符串）
        - response_time/consecutive_failures/last_state_change
        """
        enriched = dict(result or {})

        def _read_meta(key: str) -> Any:
            if meta is None:
                return None
            if hasattr(meta, key):
                return getattr(meta, key)
            if isinstance(meta, dict):
                return meta.get(key)
            return None

        def _set_default(key: str, value: Any):
            if key not in enriched or enriched.get(key) is None:
                enriched[key] = value

        window_metrics = None
        error_rate = _read_meta("window_error_rate")
        latency_p95 = _read_meta("latency_p95")
        latency_p99 = _read_meta("latency_p99")
        sample_size = _read_meta("sample_size")
        if any(v is not None for v in (error_rate, latency_p95, latency_p99, sample_size)):
            window_metrics = {
                "error_rate": error_rate,
                "latency_p95": latency_p95,
                "latency_p99": latency_p99,
                "sample_size": sample_size,
            }

        next_retry_time = _read_meta("next_retry_time")
        hard_deadline = _read_meta("hard_deadline")
        lease_deadline = _read_meta("lease_deadline")
        response_time = _read_meta("response_time") or _read_meta("last_response_time")
        consecutive_failures = _read_meta("consecutive_failures")
        last_state_change = _read_meta("last_state_change") or _read_meta("state_entered_time") or _read_meta("last_health_check")

        _set_default("status", enriched.get("status", "unknown"))
        _set_default("window_metrics", window_metrics)
        _set_default("retry_in", self._remaining_seconds(next_retry_time))
        _set_default("hard_timeout_in", self._remaining_seconds(hard_deadline))
        _set_default("lease_remaining", self._remaining_seconds(lease_deadline))
        _set_default("next_retry_time", self._serialize_time(next_retry_time))
        _set_default("hard_deadline", self._serialize_time(hard_deadline))
        _set_default("lease_deadline", self._serialize_time(lease_deadline))
        _set_default("response_time", response_time)
        _set_default("consecutive_failures", consecutive_failures)
        _set_default("last_state_change", self._serialize_time(last_state_change))
        return enriched

    def _normalize_target_statuses(self, status: Union[str, List[str]]) -> List[str]:
        """
        规范化目标状态参数

        Args:
            status: 状态参数，可以是字符串或列表

        Returns:
            List[str]: 规范化的状态列表

        Raises:
            ValueError: 当状态值无效时抛出
        """
        # 获取有效的状态值
        valid_statuses = {state.value for state in ServiceConnectionState}

        if isinstance(status, str):
            target_statuses = [status]
        elif isinstance(status, list):
            target_statuses = status
        else:
            raise ValueError(f"Status must be string or list, got {type(status)}")

        # 验证状态值
        for s in target_statuses:
            if s not in valid_statuses:
                raise ValueError(f"Invalid status '{s}'. Valid statuses are: {sorted(valid_statuses)}")

        return target_statuses

    def _apply_auth_to_update_config(self, config: Dict[str, Any],
                                    auth: Optional[str],
                                    token: Optional[str],
                                    api_key: Optional[str],
                                    headers: Optional[Dict[str, str]]) -> Dict[str, Any]:
        """将认证配置应用到更新配置中（标准化为 headers + 合并语义）"""
        final_config = config.copy() if config else {}

        # 构造标准化后的 headers
        normalized_headers: Dict[str, str] = {}
        eff_token = token if token else auth
        if eff_token:
            normalized_headers["Authorization"] = f"Bearer {eff_token}"
        if api_key:
            normalized_headers["X-API-Key"] = api_key
        if headers:
            normalized_headers.update(headers)

        if normalized_headers:
            existing = dict(final_config.get("headers", {}) or {})
            existing.update(normalized_headers)
            final_config["headers"] = existing

        # 清理入口字段，避免持久化污染
        for k in ("token", "api_key", "auth"):
            if k in final_config:
                try:
                    del final_config[k]
                except Exception:
                    final_config.pop(k, None)

        return final_config

    def _extract_service_name_from_identifier(self, client_id_or_service_name: str) -> str:
        """
        从标识符中提取服务名称（新架构辅助方法）

        Args:
            client_id_or_service_name: client_id或服务名

        Returns:
            str: 服务名称
        """
        if not isinstance(client_id_or_service_name, str):
            raise ValueError(f"Identifier must be a string, actual type: {type(client_id_or_service_name)}")

        # 如果包含client_id格式，提取服务名称
        if "::" in client_id_or_service_name:
            # global_agent_store::service_name 格式
            parts = client_id_or_service_name.split("::", 1)
            if len(parts) == 2:
                return parts[1]

        # 如果是client_id格式，提取服务名称
        if client_id_or_service_name.startswith("client_"):
            # client_global_agent_store_service_name 格式
            parts = client_id_or_service_name.split("_", 3)
            if len(parts) >= 4:
                return parts[3]

        # 直接返回作为服务名称
        return client_id_or_service_name



"""OpenAPI component classes: Tool, Resource, and ResourceTemplate."""

from __future__ import annotations

import json
import re
import warnings
from collections.abc import Callable
from typing import TYPE_CHECKING, Any

import httpx
from mcp.types import ToolAnnotations
from mcpstore.mcp.utilities.openapi import HTTPRoute
from mcpstore.mcp.utilities.openapi.director import RequestDirector
from pydantic.networks import AnyUrl

from mcpstore.mcp.resources import (
    Resource,
    ResourceContent,
    ResourceResult,
    ResourceTemplate,
)
from mcpstore.mcp.server.dependencies import get_http_headers
from mcpstore.mcp.server.tasks.config import TaskConfig
from mcpstore.mcp.tools.tool import Tool, ToolResult
from mcpstore.mcp.utilities.logging import get_logger

if TYPE_CHECKING:
    from mcpstore.mcp.server import Context

__all__ = [
    "OpenAPIResource",
    "OpenAPIResourceTemplate",
    "OpenAPITool",
]

logger = get_logger(__name__)


def _slugify(text: str) -> str:
    """Convert text to a URL-friendly slug format.

    Only contains lowercase letters, uppercase letters, numbers, and underscores.
    """
    if not text:
        return ""

    # Replace spaces and common separators with underscores
    slug = re.sub(r"[\s\-\.]+", "_", text)

    # Remove non-alphanumeric characters except underscores
    slug = re.sub(r"[^a-zA-Z0-9_]", "", slug)

    # Remove multiple consecutive underscores
    slug = re.sub(r"_+", "_", slug)

    # Remove leading/trailing underscores
    slug = slug.strip("_")

    return slug


class OpenAPITool(Tool):
    """Tool implementation for OpenAPI endpoints."""

    task_config: TaskConfig = TaskConfig(mode="forbidden")

    def __init__(
        self,
        client: httpx.AsyncClient,
        route: HTTPRoute,
        director: RequestDirector,
        name: str,
        description: str,
        parameters: dict[str, Any],
        output_schema: dict[str, Any] | None = None,
        tags: set[str] | None = None,
        timeout: float | None = None,
        annotations: ToolAnnotations | None = None,
        serializer: Callable[[Any], str] | None = None,  # Deprecated
    ):
        if serializer is not None and settings.deprecation_warnings:
            warnings.warn(
                "The `serializer` parameter is deprecated. "
                "Return ToolResult from your tools for full control over serialization. "
                "See https://gomcpstore.mcp.com/servers/tools#custom-serialization for migration examples.",
                DeprecationWarning,
                stacklevel=2,
            )
        super().__init__(
            name=name,
            description=description,
            parameters=parameters,
            output_schema=output_schema,
            tags=tags or set(),
            annotations=annotations,
            serializer=serializer,
        )
        self._client = client
        self._route = route
        self._director = director
        self._timeout = timeout

    def __repr__(self) -> str:
        return f"OpenAPITool(name={self.name!r}, method={self._route.method}, path={self._route.path})"

    async def run(self, arguments: dict[str, Any]) -> ToolResult:
        """Execute the HTTP request using RequestDirector."""
        try:
            base_url = str(self._client.base_url) or "http://localhost"

            # Build the request using RequestDirector
            request = self._director.build(self._route, arguments, base_url)

            # Add client headers (lowest precedence)
            if self._client.headers:
                for key, value in self._client.headers.items():
                    if key not in request.headers:
                        request.headers[key] = value

            # Add MCP transport headers (highest precedence)
            mcp_headers = get_http_headers()
            if mcp_headers:
                request.headers.update(mcp_headers)

            logger.debug(f"run - sending request; headers: {request.headers}")

            response = await self._client.send(request)
            response.raise_for_status()

            # Try to parse as JSON first
            try:
                result = response.json()

                # Handle structured content based on output schema
                if self.output_schema is not None:
                    if self.output_schema.get("x-mcpstore-wrap-result"):
                        structured_output = {"result": result}
                    else:
                        structured_output = result
                elif not isinstance(result, dict):
                    structured_output = {"result": result}
                else:
                    structured_output = result

                return ToolResult(structured_content=structured_output)
            except json.JSONDecodeError:
                return ToolResult(content=response.text)

        except httpx.HTTPStatusError as e:
            error_message = (
                f"HTTP error {e.response.status_code}: {e.response.reason_phrase}"
            )
            try:
                error_data = e.response.json()
                error_message += f" - {error_data}"
            except (json.JSONDecodeError, ValueError):
                if e.response.text:
                    error_message += f" - {e.response.text}"
            raise ValueError(error_message) from e

        except httpx.RequestError as e:
            raise ValueError(f"Request error: {e!s}") from e


class OpenAPIResource(Resource):
    """Resource implementation for OpenAPI endpoints."""

    task_config: TaskConfig = TaskConfig(mode="forbidden")

    def __init__(
        self,
        client: httpx.AsyncClient,
        route: HTTPRoute,
        director: RequestDirector,
        uri: str,
        name: str,
        description: str,
        mime_type: str = "application/json",
        tags: set[str] | None = None,
        timeout: float | None = None,
    ):
        super().__init__(
            uri=AnyUrl(uri),
            name=name,
            description=description,
            mime_type=mime_type,
            tags=tags or set(),
        )
        self._client = client
        self._route = route
        self._director = director
        self._timeout = timeout

    def __repr__(self) -> str:
        return f"OpenAPIResource(name={self.name!r}, uri={self.uri!r}, path={self._route.path})"

    async def read(self) -> ResourceResult:
        """Fetch the resource data by making an HTTP request."""
        try:
            path = self._route.path
            resource_uri = str(self.uri)

            # If this is a templated resource, extract path parameters from the URI
            if "{" in path and "}" in path:
                parts = resource_uri.split("/")

                if len(parts) > 1:
                    path_params = {}
                    param_matches = re.findall(r"\{([^}]+)\}", path)
                    if param_matches:
                        param_matches.sort(reverse=True)
                        expected_param_count = len(parts) - 1
                        for i, param_name in enumerate(param_matches):
                            if i < expected_param_count:
                                param_value = parts[-1 - i]
                                path_params[param_name] = param_value

                    for param_name, param_value in path_params.items():
                        path = path.replace(f"{{{param_name}}}", str(param_value))

            # Build headers with correct precedence
            headers: dict[str, str] = {}
            if self._client.headers:
                headers.update(self._client.headers)
            mcp_headers = get_http_headers()
            if mcp_headers:
                headers.update(mcp_headers)

            response = await self._client.request(
                method=self._route.method,
                url=path,
                headers=headers,
                timeout=self._timeout,
            )
            response.raise_for_status()

            content_type = response.headers.get("content-type", "").lower()

            if "application/json" in content_type:
                result = response.json()
                return ResourceResult(
                    contents=[
                        ResourceContent(
                            content=json.dumps(result), mime_type="application/json"
                        )
                    ]
                )
            elif any(ct in content_type for ct in ["text/", "application/xml"]):
                return ResourceResult(
                    contents=[
                        ResourceContent(content=response.text, mime_type=self.mime_type)
                    ]
                )
            else:
                return ResourceResult(
                    contents=[
                        ResourceContent(
                            content=response.content, mime_type=self.mime_type
                        )
                    ]
                )

        except httpx.HTTPStatusError as e:
            error_message = (
                f"HTTP error {e.response.status_code}: {e.response.reason_phrase}"
            )
            try:
                error_data = e.response.json()
                error_message += f" - {error_data}"
            except (json.JSONDecodeError, ValueError):
                if e.response.text:
                    error_message += f" - {e.response.text}"
            raise ValueError(error_message) from e

        except httpx.RequestError as e:
            raise ValueError(f"Request error: {e!s}") from e


class OpenAPIResourceTemplate(ResourceTemplate):
    """Resource template implementation for OpenAPI endpoints."""

    task_config: TaskConfig = TaskConfig(mode="forbidden")

    def __init__(
        self,
        client: httpx.AsyncClient,
        route: HTTPRoute,
        director: RequestDirector,
        uri_template: str,
        name: str,
        description: str,
        parameters: dict[str, Any],
        tags: set[str] | None = None,
        timeout: float | None = None,
    ):
        super().__init__(
            uri_template=uri_template,
            name=name,
            description=description,
            parameters=parameters,
            tags=tags or set(),
        )
        self._client = client
        self._route = route
        self._director = director
        self._timeout = timeout

    def __repr__(self) -> str:
        return f"OpenAPIResourceTemplate(name={self.name!r}, uri_template={self.uri_template!r}, path={self._route.path})"

    async def create_resource(
        self,
        uri: str,
        params: dict[str, Any],
        context: Context | None = None,
    ) -> Resource:
        """Create a resource with the given parameters."""
        uri_parts = [f"{key}={value}" for key, value in params.items()]

        return OpenAPIResource(
            client=self._client,
            route=self._route,
            director=self._director,
            uri=uri,
            name=f"{self.name}-{'-'.join(uri_parts)}",
            description=self.description or f"Resource for {self._route.path}",
            mime_type="application/json",
            tags=set(self._route.tags or []),
            timeout=self._timeout,
        )
